/*
 *  Router.java
 *
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.bloom;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;
import java.util.Random;
import java.util.LinkedList;
import java.io.FileInputStream;
import mdw.sandStorm.api.ConfigDataIF;
import mdw.sandStorm.api.EventHandlerException;
import mdw.sandStorm.api.EventHandlerIF;
import mdw.sandStorm.api.QueueElementIF;
import mdw.sandStorm.api.SinkIF;
import mdw.sandStorm.api.SinkException;
import mdw.sandStorm.api.StagesInitializedSignal;
import mdw.sandStorm.api.NoSuchStageException;
import ostore.util.QSIO;
import ostore.util.QSByteArray;
import ostore.util.Carp;
import ostore.util.Clock;
import ostore.util.Assert;
import ostore.util.ByteUtils;
import ostore.util.Debug;
import ostore.util.DebugFlags;
import ostore.util.SecureHash;
import ostore.util.SHA1Hash;
import ostore.util.NodeId;
import ostore.util.TypeTable;
import ostore.util.Types;
import ostore.util.QSException;
import ostore.util.QuickSerializable;
import ostore.util.Pair;
import ostore.dispatch.Classifier;
import ostore.dispatch.Filter;
import ostore.tapestry.api.TapestryRouteMsg;
import ostore.tapestry.api.TapestryMacRouteMsg;
import ostore.tapestry.api.TapestryPrefixRouteMsg;
import ostore.tapestry.api.TapestryInstance;
import ostore.tapestry.api.TapestryTag;
import ostore.tapestry.api.TapestryLocateMsg;
import ostore.tapestry.api.TapestryLocateFailure;
import ostore.tapestry.api.TapestryPublishMsg;
import ostore.tapestry.api.TapestryQuery;
import ostore.tapestry.api.TapestryQueryState;
import ostore.tapestry.api.TapestryQueryResult;
import ostore.tapestry.api.TapestryQueryResultMatch;
import ostore.tapestry.api.TapestryUnpublishMsg;
import ostore.tapestry.impl.RouteMsg;
import ostore.tapestry.impl.RouteObjRootMsg;
import ostore.tapestry.impl.RouteObjDestMsg;
import ostore.tapestry.impl.PublishMsg;
import ostore.tapestry.impl.BloomWrapper;
import ostore.tapestry.impl.StaticAliveMsg;
import ostore.tapestry.impl.StaticAllNodesMsg;
import ostore.tapestry.impl.StaticBeginMsg;
import ostore.tapestry.impl.StaticReadyMsg;
import ostore.tapestry.impl.StaticPingMsg;
import ostore.tapestry.impl.StaticPongMsg;
import ostore.bloom.BloomQuery;

/**
 * Fake Tapestry for simulation purposes.
 *
 * @author     Sean C. Rhea
 * @version    $Id: HomeNode.java,v 1.5 2002/03/27 03:52:51 srhea Exp $
 */
public class HomeNode implements EventHandlerIF {

    private String class_tag = "HomeNode";
    private static final boolean DEBUG = DebugFlags.BLOOM;

    private void dispatch (QueueElementIF item) {
	final String tag = class_tag + ".dispatch";
	if (DEBUG) Debug.printtagln (tag, "dispatching " +
		item.getClass () + " event: " + item + ".");
	try {
	    Classifier.getClassifier(_self_node_id).dispatch(item);
	} catch (SinkException e) {
	    BUG (class_tag + ".dispatch: could not dispatch " + item);
	}
    }

    private static final void BUG (String msg) {
	Carp.die (msg + "  This is a bug in " +
		"ostore.bloom.HomeNode.  " +
		"Please let Sean know about it ASAP.");
    }

    private static Vector _all_nodes = new Vector ();
    private static Map _all_nodes_map = new HashMap ();
    private static Map _guid_to_node_id = new HashMap ();
    private static Map _home_nodes = new HashMap ();
    private static Random _rand;

    public void init (ConfigDataIF config) throws Exception {
	_self_node_id = new NodeId (config.getString ("node_id"));
	
	if (DEBUG) Debug.turnOn (class_tag);
	class_tag = class_tag + ".[" + _self_node_id.port () + "]";
	final String tag = class_tag + ".init";

	Types.registerAllTypes ();

	_using_bloom = config.getBoolean ("using_bloom");
	int seed = config.getInt ("seed");
	Assert.assert (seed > 0, tag + "seed <= 0");
	if (_rand == null) _rand = new Random (seed);
	_sink = config.getStage().getSink();
	
	_self_guid = new SHA1Hash (_self_node_id);

	_all_nodes.add (_self_node_id);
	_all_nodes_map.put (_self_node_id, this);
	_guid_to_node_id.put (_self_guid, _self_node_id);

	_timing_map = null;
	_out_times = null;
	_fed_node_id = new NodeId (config.getString ("federation"));
	_waiting_reqs = new LinkedList ();
	_running = false;

	System.out.println (tag + ": Home Node ID is " + _self_guid + ".");
	TapestryInstance.instance (_self_node_id).set_self_guid (_self_guid);

	String [] event_types = {
	    "mdw.sandStorm.api.StagesInitializedSignal",
	    "ostore.tapestry.api.TapestryPublishMsg",
	    "ostore.tapestry.api.TapestryUnpublishMsg"
	};

	for (int i = 0; i < event_types.length; ++i) {
	    Filter filter = new Filter ();
	    if (! filter.requireType (Class.forName (event_types [i])))
		BUG (tag + ": could not require type " + event_types [i]);
	    Classifier.getClassifier(_self_node_id).subscribe (filter, _sink);
	}

	String [] inb_msg_types = {
	    "ostore.tapestry.impl.RouteMsg",
	    "ostore.tapestry.impl.RouteObjRootMsg",
	    "ostore.tapestry.impl.RouteObjDestMsg",
	    "ostore.tapestry.impl.PublishMsg",
	    "ostore.tapestry.impl.BloomWrapper",
	    "ostore.tapestry.impl.StaticAllNodesMsg",
	    "ostore.tapestry.impl.StaticBeginMsg",
	    "ostore.tapestry.impl.StaticPingMsg",
	    "ostore.tapestry.impl.StaticPongMsg"
	};

	for (int i = 0; i < inb_msg_types.length; ++i) {
	    Filter filter = new Filter ();
	    if (! filter.requireType (Class.forName (inb_msg_types [i])))
		BUG (tag + ": could not require type " + inb_msg_types [i]);
	    if (! filter.requireValue ("inbound", new Boolean (true)))
		BUG (tag + ": could not require inbound = true for " 
			+ inb_msg_types [i]);
	    Classifier.getClassifier(_self_node_id).subscribe (filter, _sink);
	}

	String [] outb_msg_types = {
	    "ostore.tapestry.api.TapestryRouteMsg",
	    "ostore.tapestry.api.TapestryLocateMsg"
	};

	for (int i = 0; i < outb_msg_types.length; ++i) {
	    Filter filter = new Filter ();
	    if (! filter.requireType (Class.forName (outb_msg_types [i])))
		BUG (tag + ": could not require type " + outb_msg_types [i]);
	    if (! filter.requireValue ("inbound", new Boolean (false)))
		BUG (tag + ": could not require inbound = false for " 
			+ outb_msg_types [i]);
	    Classifier.getClassifier(_self_node_id).subscribe (filter, _sink);
	}
    }

    public void destroy() throws Exception {
	// TODO
    }

    /**
     * For use in derived classes.
     */
    protected void handle_stages_init_signal () {
	final String tag = class_tag + ".handle_stages_init_signal";

	if (DEBUG) Debug.printtagln (tag,
		"processing StagesInitializedSignal" );

	StaticAliveMsg alive_msg = new StaticAliveMsg (
		_fed_node_id, _self_guid);
	dispatch (alive_msg);
    }

    protected void handle_static_all_nodes_msg (StaticAllNodesMsg msg) {
	final String tag = class_tag + ".handle_static_all_nodes_msg";
	if (DEBUG) Debug.printtagln (tag, "processing StaticAllNodesMsg" );
	if (_my_node_map != null) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got duplicate StaticAllNodesMsg: " +
		    msg + ".  Dropping it.");
	    return;
	}

	if (! msg.peer.equals (_fed_node_id)) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got StaticAllNodesMsg: " + msg + 
		    " from someone other than the federation.  " +
		    "Dropping it.");
	    return;
	}

	_my_node_map = msg.nodes ();
	_out_times = new HashMap ();
	_timing_map = new HashMap ();

	for (Iterator i = _my_node_map.keySet ().iterator (); 
		i.hasNext (); ) {
	    NodeId n = (NodeId) i.next ();
	    if (! n.equals (_self_node_id)) {
		StaticPingMsg ping_msg = new StaticPingMsg (n);
		_out_times.put (n, new Long (
			    Clock.current_date (_self_node_id).getTime ()));
		dispatch (ping_msg);
	    }
	}
    }
    protected void handle_static_ping_msg (StaticPingMsg msg) {
	final String tag = class_tag + ".handle_static_ping_msg";
	if (DEBUG) Debug.printtagln (tag, "processing StaticPingMsg" );
	if (! _running)
	    dispatch (new StaticPongMsg (msg.peer));
    }

    protected void handle_static_pong_msg (StaticPongMsg msg) {
	final String tag = class_tag + ".handle_static_pong_msg";
	if (DEBUG) Debug.printtagln (tag, "processing StaticPongMsg" );
	if ((_out_times == null) || (_out_times.isEmpty ())) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got duplicate StaticPongMsg: " +
		    msg + ".  Dropping it.");
	    return;
	}

	long recv_time = Clock.current_date (_self_node_id).getTime ();

	if (! _out_times.containsKey (msg.peer)) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got StaticPongMsg: " + msg + 
		    " from someone unexpected.  " +
		    "Dropping it.");
	    return;
	}

	long send_time = 
	    ((Long) _out_times.remove (msg.peer)).longValue ();
	long delta = recv_time - send_time;

	_timing_map.put (msg.peer, new Long (delta));

	if (DEBUG) Debug.printtagln (tag, 
		"_out_times.size () = " + _out_times.size ());

	if (_out_times.isEmpty ()) {
	    // The timing map is complete.  Fill in our routing tables
	    // and send a ready message.

	    _out_times = null;
	    dispatch (new StaticReadyMsg (_fed_node_id));
	}
    }

    protected void handle_static_begin_msg (StaticBeginMsg msg) 
    throws EventHandlerException {
	final String tag = class_tag + ".handle_static_begin_msg";
	if (DEBUG) Debug.printtagln (tag, "processing StaticBeginMsg" );

	if (_running) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got duplicate StaticBeginMsg: " +
		    msg + ".  Dropping it.");
	    return;
	}

	if (! msg.peer.equals (_fed_node_id)) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got StaticAllNodesMsg: " + msg + 
		    " from someone other than the federation.  " +
		    "Dropping it.");
	    return;
	}

	_running = true;
	try {
	    while (! _waiting_reqs.isEmpty ()) {
		QueueElementIF e = 
		    (QueueElementIF) _waiting_reqs.removeFirst ();
		handleEvent (e);
	    }
	}
	catch (EventHandlerException e) {
	    BUG (tag + ": what do I do now?"); // TODO
	}
    }


    protected void handle_tapestry_route_msg (TapestryRouteMsg msg) {
	final String tag = class_tag + ".handle_tapestry_route_msg";
	if (DEBUG) Debug.printtagln (tag, "processing TapestryRouteMsg " + msg);

	NodeId dst = (NodeId) _guid_to_node_id.get (msg.peer);
	RouteMsg outb = 
	    new RouteMsg (dst, _self_guid, msg.peer, false, 0, msg);
	dispatch (outb);
    }

    protected void handle_route_msg (RouteMsg msg) {
	final String tag = class_tag + ".handle_route_msg";

	Assert.assert (msg.dest.equals (_self_guid), ": oops!");

	TapestryRouteMsg inb = null;
	try {
	    inb = (TapestryRouteMsg) msg.user_data ();
	}
	catch (ClassCastException e) {
	    BUG (tag + ": should have caught this above.");
	    return;
	}
	catch (QSException e) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + "caught " + e + " decoding "
		    + msg + ".  Dropping it.");
	    return;
	}
	inb.peer = msg.src;
	inb.inbound = true;

	dispatch (inb);
    }

    private static NodeId compute_home_node (SecureHash guid) {
	NodeId home_node = null;
	Assert.assert (guid != null, "Guid is null.");
	synchronized (_home_nodes) {
	    home_node = (NodeId) _home_nodes.get (guid);
	    if (home_node == null) {
		int choice = (_rand.nextInt () & 0x7fffffff) 
		    % _all_nodes.size ();
		home_node = (NodeId) _all_nodes.elementAt (choice);
		_home_nodes.put (guid, home_node);
	    }
	}
	return home_node;
    }

    protected void handle_tapestry_publish_msg (TapestryPublishMsg msg) {
	final String tag = class_tag + ".handle_tapestry_publish_msg";
	if (DEBUG) 
	    Debug.printtagln (tag, "processing TapestryPublishMsg " + msg);
	_published_guids.add (msg.guid);
	NodeId home_node = compute_home_node (msg.guid);
	if (home_node.equals (_self_node_id)) {
	    publish (msg.guid, _self_node_id, true);
	}
	else {
	    PublishMsg outb = 
		new PublishMsg (home_node, _self_guid, msg.guid, null, 0, true);
	    dispatch (outb);
	}
    }

    protected void handle_tapestry_unpublish_msg (TapestryUnpublishMsg msg) {
	final String tag = class_tag + ".handle_tapestry_unpublish_msg";
	if (DEBUG) 
	    Debug.printtagln(tag, "processing TapestryUnpublishMsg "+ msg);
	_published_guids.remove (msg.guid);
	NodeId home_node = compute_home_node (msg.guid);
	if (home_node.equals (_self_node_id)) {
	    publish (msg.guid, _self_node_id, false);
	}
	else {
	    PublishMsg outb = new PublishMsg (
		    home_node, _self_guid, msg.guid, null, 0, false);
	    dispatch (outb);
	}
    }

    protected void publish (SecureHash guid, NodeId replica, boolean publish) {
	Set replicas = (Set) _replica_map.get (guid);
	if (replicas == null) {
	    replicas = new HashSet ();
	    _replica_map.put (guid, replicas);
	}
	if (publish)
	    replicas.add (replica);
	else
	    replicas.remove (replica);
    }

    protected void handle_publish_msg (PublishMsg msg) {
	final String tag = class_tag + ".handle_publish_msg";
	if (DEBUG) Debug.printtagln (tag, "processing PublishMsg " + msg);
	publish (msg.guid, msg.peer, msg.publish);
    }

    protected void handle_bloom_wrapper (BloomWrapper wrapper) {
	final String tag = class_tag + ".handle_bloom_wrapper";
	if (wrapper.success) {
	    if (wrapper.user_data instanceof TapestryLocateMsg) {
		TapestryLocateMsg inb = 
		    (TapestryLocateMsg) ByteUtils.qs_clone (wrapper.user_data);

		inb.peer = wrapper.src;
		inb.inbound = true;
		inb.guid = wrapper.dest;
		inb.query = null;
		inb.query_state = null;

		dispatch (inb);
	    }
	    else if (wrapper.user_data instanceof TapestryRouteMsg) {
		TapestryRouteMsg inb = 
		    (TapestryRouteMsg) ByteUtils.qs_clone (wrapper.user_data);

		inb.peer = wrapper.src;
		inb.inbound = true;

		dispatch (inb);
	    }
	    else {
		BUG (tag + ": don't know what to do with " + 
			wrapper.user_data + ".");
	    }
	}
	else {
	    if (wrapper.user_data instanceof TapestryLocateMsg) {

		NodeId home_node = compute_home_node (wrapper.dest);
		RouteObjRootMsg r = new RouteObjRootMsg (home_node, 
			wrapper.src, wrapper.dest, null, null, null, -1, 
			wrapper.user_data, null);
		if (home_node.equals (_self_node_id)) {
		    handle_route_obj_root_msg (r);
		}
		else {
		    dispatch (r);
		}
	    }
	    else {
		BUG (tag + ": don't know what to do with " + 
			wrapper.user_data + ".");
	    }
	}
    }

    protected void handle_tapestry_locate_msg (TapestryLocateMsg msg) {
	final String tag = class_tag + ".handle_tapestry_locate_msg";
	if (DEBUG) Debug.printtagln (tag, 
		"processing TapestryLocateMessage "+ msg);

	if (_using_bloom && (msg.query == null) && (msg.id == null)) {
	    dispatch (new BloomWrapper (msg.guid, msg));
	    return;
	}

	if (_published_guids.contains (msg.guid)) {
	    TapestryLocateMsg inb = 
		(TapestryLocateMsg) ByteUtils.qs_clone (msg);
	    inb.peer = _self_guid;
	    inb.inbound = true;
	    inb.guid = msg.guid;
	    inb.query = null;
	    inb.query_state = null;
	    dispatch (inb);
	    return;
	}

	NodeId home_node = compute_home_node (msg.guid);
	RouteObjRootMsg outb = new RouteObjRootMsg (
		home_node, _self_guid, msg.guid, msg.query,
		msg.query_state, msg.id, 0, msg, null);
	if (home_node.equals (_self_node_id)) {
	    handle_route_obj_root_msg (outb);
	}
	else {
	    dispatch (outb);
	}
    }

    protected void handle_route_obj_root_msg (RouteObjRootMsg msg) {
	final String tag = class_tag + ".handle_route_obj_root_msg";
	if (DEBUG) Debug.printtagln (tag, "processing RouteObjRootMsg " + msg);

	NodeId home_node = compute_home_node (msg.guid);
	Assert.assert (home_node.equals (_self_node_id), "Shite");
	Set replicas = (Set) _replica_map.get (msg.guid);

	// TODO: pick closest
	HomeNode other = (HomeNode) _all_nodes_map.get (msg.peer);
	NodeId closest = null;
	long min_dist = Long.MAX_VALUE;
	for (Iterator i = replicas.iterator (); i.hasNext (); ) {
	    NodeId replica = (NodeId) i.next ();
	    long d = ((Long) other._timing_map.get (replica)).longValue ();
	    if (d < min_dist) {
		min_dist = d;
		closest = replica;
	    }
	}
	RouteObjDestMsg outb = new RouteObjDestMsg (closest, msg.src,
		_dummy_hash, msg.guid, 0, 
		msg._user_data_length,
		msg._user_data_offset,
		msg._user_data_type,
		msg._user_data_buffer);
	if (closest.equals (_self_node_id)) {
	    handle_route_obj_dest_msg (outb);
	}
	else {
	    dispatch (outb);
	}
    }

    public void handle_route_obj_dest_msg (RouteObjDestMsg msg) {
	final String tag = class_tag + ".handle_route_obj_dest_msg";
	if (DEBUG) Debug.printtagln (tag, "processing RouteObjDestMsg " + msg);
	TapestryLocateMsg inb = null;
	try {
	    inb = (TapestryLocateMsg) msg.user_data ();
	}
	catch (QSException e) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + "caught " + e + 
		    " decoding " + msg + ".  Dropping it...");
	    return;
	}
	inb.peer = msg.src;
	inb.inbound = true;
	inb.guid = msg.guid;
	inb.query = null;
	inb.query_state = null;
	dispatch (inb);
    }

    public void handleEvent(QueueElementIF item) 
	throws EventHandlerException {
        final String tag = class_tag + ".handleEvent";

	/*if (DEBUG) Debug.printtagln (tag, "got " + item.getClass () + 
		    " event: " + item + ".");*/

	if (item instanceof StagesInitializedSignal) {
	    handle_stages_init_signal ();
	}
	else if (item instanceof BloomWrapper) {
	    handle_bloom_wrapper ((BloomWrapper) item);
	}
	else if (item instanceof StaticAllNodesMsg) {
	    handle_static_all_nodes_msg ((StaticAllNodesMsg) item);
	}
	else if (item instanceof StaticPingMsg) {
	    handle_static_ping_msg ((StaticPingMsg) item);
	}
	else if (item instanceof StaticPongMsg) {
	    handle_static_pong_msg ((StaticPongMsg) item);
	}
	else if (item instanceof StaticBeginMsg) {
	    handle_static_begin_msg ((StaticBeginMsg) item);
	}
	else if (! _running) {
	    _waiting_reqs.addLast (item);
	    return;
	}
	else {
	    if (item instanceof TapestryRouteMsg) {
		handle_tapestry_route_msg ((TapestryRouteMsg) item);
	    }
	    else if (item instanceof RouteMsg) {
		handle_route_msg ((RouteMsg) item);
	    }
	    else if (item instanceof TapestryPublishMsg) {
		handle_tapestry_publish_msg ((TapestryPublishMsg) item);
	    }
	    else if (item instanceof TapestryUnpublishMsg) {
	        handle_tapestry_unpublish_msg ((TapestryUnpublishMsg) item);
	    }
	    else if (item instanceof PublishMsg) {
		handle_publish_msg ((PublishMsg) item);
	    }
	    else if (item instanceof TapestryLocateMsg) {
		handle_tapestry_locate_msg ((TapestryLocateMsg) item);
	    }
	    else if (item instanceof RouteObjRootMsg) {
		handle_route_obj_root_msg ((RouteObjRootMsg) item);
	    }
	    else if (item instanceof RouteObjDestMsg) {
		handle_route_obj_dest_msg ((RouteObjDestMsg) item);
	    }
	}
    }

    public void handleEvents(QueueElementIF items[]) 
    throws EventHandlerException {
	for (int i=0; i < items.length; i++)
	    handleEvent (items[i]);
    }

    private Map /* SecureHash -> NodeId */ _replica_map = new HashMap ();
    private NodeId _self_node_id;
    private SecureHash _self_guid;
    private SinkIF _sink;
    private boolean _using_bloom;
    private NodeId _fed_node_id;
    private boolean _running;
    private Map _timing_map;
    private Map _out_times;
    private Map _my_node_map;
    private LinkedList _waiting_reqs;
    private Set _published_guids = new HashSet ();

    private static final SecureHash _dummy_hash = new SHA1Hash ("foo");
}

