/**
 * DHMessage
 *
 * @author    Gifford Cheung
 * @version     $Id: DHMessage.java, v 1.0 2001/06/14 12:01:00 gkcheung Exp
 */

package ostore.mac;

import ostore.tapestry.*;
import ostore.util.*;
import ostore.security.*;
import mdw.sandStorm.api.*;
import javax.crypto.interfaces.*;

/**
 * Message specific to Diffie-Hellman StS key agreement <br></br>
 */
public class DHMessage extends RouteTMessage implements QuickSerializable{
    
    public static int INITIALIZE    = 0;
    public static int RESPOND       = 1;
    public static int FINALIZE      = 2;

    public QSDHPublicKey QSkey;
    public QSPublicKey sigkey;
    public QSMACType MACType;
    QuickSerializable user_data;
    QSByteArray mac;
    public int messageType; // one of INITIALIZE, RESPOND, FINALIZE

    public DHMessage(SecureHash toWhom,
                     DHPublicKey key,
                     QSPublicKey sigkey,
                     QSMACType MACType,
		     QuickSerializable user_data,
		     QSByteArray mac,
                     int messageType ) {
        super(toWhom, true);
	this.usingMAC = false; // infinite loops are bad
        
        if (key != null) this.QSkey = new QSDHPublicKey(key);
        this.sigkey = sigkey;
        this.MACType = MACType;
        this.user_data = user_data;
        this.mac = mac;
        this.messageType = messageType;
    }

    /** 
     *  Generate DHMessage from data
     */
      public DHMessage (byte [] data, int [] offset) throws QSException {
        super(data, offset);
        // QSkey
        boolean QSkeyExists = ByteUtils.bytesToBool( data, offset);
        if (QSkeyExists) {
            QSkey = new QSDHPublicKey(data, offset);
        }
        // MACType
        int mtype = ByteUtils.bytesToInt( data, offset);
        MACType = (QSMACType) TypeTable.new_object(mtype, data, offset);
        // sigkey
        boolean skExists = ByteUtils.bytesToBool( data, offset);
        if (skExists) {
            int stype = ByteUtils.bytesToInt( data, offset);
            sigkey = (QSPublicKey) TypeTable.new_object( stype, data, offset);
        }
        // verify
        boolean vExists = ByteUtils.bytesToBool( data, offset);
        if (vExists)  {
	    int type = ByteUtils.bytesToInt( data, offset);
	    user_data = TypeTable.new_object( type, data, offset);
	    mac = new QSByteArray( data, offset);
        }
        // messageType
        messageType = ByteUtils.bytesToInt (data, offset);
      }

      public void to_bytes (byte [] data, int [] offset) {
        // QSkey
        ByteUtils.boolToBytes( (QSkey != null), data, offset);
        if (QSkey != null) {
            QSkey.to_bytes( data,offset);
        }
        // MACType
        ByteUtils.intToBytes( MACType.type_code(), data, offset);
        MACType.to_bytes( data, offset);
        // sigkey
        ByteUtils.boolToBytes( (sigkey != null), data, offset);
        if (sigkey != null) {
            ByteUtils.intToBytes( sigkey.type_code(), data, offset);
            sigkey.to_bytes( data, offset);
        }
        // verify
        ByteUtils.boolToBytes( (mac != null), data, offset);
        if (mac != null) {
            ByteUtils.intToBytes( user_data.type_code(), data, offset);
            user_data.to_bytes( data, offset);
            mac.to_bytes( data, offset);
        }
        // messageType
        ByteUtils.intToBytes(messageType, data, offset);
      }

    public int type_code() {
        return Types.MAC_DHMESSAGE;
    }

    public void print() {
        byte[] temp;
        System.out.println("Printing a DHMessage");
        if (QSkey != null) {
            temp = ((DHPublicKey)QSkey.key()).getY().toByteArray();
            System.out.println("(*)QS Public Diffie Hellman: " + 
                                ByteUtils.print_bytes(temp, 0, temp.length) );
        } else {
            System.out.println("(*)QS Public Diffie Hellman: null");
        }

        if (sigkey != null) {
            System.out.println("(*)QS signature key: " + 
                                new SHA1Hash (sigkey));
        } else {
            System.out.println("(*)QS signature key: null");
        }

        if (MACType != null) {
            System.out.println("(*)MAC Type: " + MACType.algorithm());
        } else {
            System.out.println("(*)MAC Type: null");
        }

	// just print it out
        System.out.println("(*)Verify: " + user_data + " " + mac); 
        
        switch (messageType) {
            case 0: System.out.println("(*)Message Type: INITIALIZE"); break;
            case 1: System.out.println("(*)Message Type: RESPOND"); break;
            case 2: System.out.println("(*)Message Type: FINALIZE"); break;
        }
    }
        
}
