/**
 * FakeClient.
 *
 * @author	Gifford Cheung
 */  

package ostore.mac;
import java.util.*;
import java.net.*;
import ostore.util.*;
import ostore.dispatch.*;
import ostore.tapestry.*;
import ostore.startrek.*;
import ostore.network.*;
import mdw.sandStorm.api.*;
import mdw.sandStorm.core.*;
import mdw.sandStorm.internal.SinkProxy;

/**
 * FakeClient stage. Sends out messages that are meant to be MAC'd or
 * sends out MAC
 */
public class FakeClient implements EventHandlerIF, ProfilableIF {

	public static final boolean DEBUG = true;
	private int _x = 0;
	private String _msgType;

	public void handleEvent(QueueElementIF item)
		throws EventHandlerException {

		// dispatch item and send message to myself 
		// as per the startrek Test style

		Carp.out ("FakeClient sends a message out...");
		try {
			Classifier.getClassifier().dispatch(item);
		} catch (SinkException e) {
			Carp.die( "FakeClient: sink exception. " + e );
		}
		next_message();

	}

	public void handleEvents(QueueElementIF item[])
		throws EventHandlerException {

		for (int i = 0; i < item.length; i += 1)
			handleEvent(item[i]);

	}

	public void init(ConfigDataIF config) throws Exception {

		System.err.println("initializing fake client stage.");

		_msgType = config.getString("msgType");

		String stageName = config.getStage().getName();
		try 
		{
		  if(config.getBoolean("Profile"))
		  {
		    config.getManager().getProfiler().add("ostore.mac.FakeClient", this);
		    SinkIF sink =
		      config.getManager().getStage(stageName).getSink();
		    _this_sink = new SinkProxy(sink,
					  config.getManager(),
					  config.getStage().getWrapper());
		  }
		  else
		    _this_sink =
		      config.getManager().getStage(stageName).getSink();
		} 
		catch (NoSuchStageException e) 
		{
		  Carp.die (stageName + ".init: could not get own sink.");
		}

		_timer = new ssTimer();
		_sequence = 0;

		next_message();

	}

	public void next_message() {
		RouteTMessage msg = null;

		if (_msgType.equals("MACedQS")) {
			QSString stub = new QSString("Msg " + _x++);
			MACedQS x = new MACedQS(stub);
			msg = new MACMessage(new SHA1Hash(), true, x);
			_timer.registerEvent (15000, msg, _this_sink);
		} 
		if (_msgType.equals("Generic")) {	
			msg =
            		  new GenericTapestryRouteMessage(_sequence++, null, true);
			msg.verifiedMAC = true;
			msg.inbound = false;
			_timer.registerEvent (15000, msg, _this_sink);
		}
	}
		

	public void destroy() throws Exception {
		//TODO
	}

        //  methods for ProfilableIF
        /* Specified by mdw.sandStorm.api.ProfilableIF */
        public int profileSize()
        {
	    return _this_sink.size();
	}
        //  end methods for ProfilableIF

	private SinkIF _this_sink;
	private ssTimer _timer;
	private int _sequence;

}
				
