/**
 * StageTester.
 *
 * @author    Gifford Cheung
 * @version    $Id: StageTester.java,v 1.8 2001/11/30 03:34:44 hweather Exp $
 */  

package ostore.mac;
import java.util.*;
import java.io.*;
import ostore.util.*;
import ostore.dispatch.*;
import ostore.security.*;
import mdw.sandStorm.api.*;
import mdw.sandStorm.core.*;
import mdw.sandStorm.internal.SinkProxy;

// Security stuff
import java.security.*;
import java.security.spec.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;
import cryptix.jce.provider.*;
import cryptix.jce.provider.random.*;
import cryptix.jce.provider.key.*;

/**
 * StageTester allows me (Gifford) to test a stage and watch it for
 * effects.
 */
public class StageTester implements EventHandlerIF, ProfilableIF {

    public static final boolean DEBUG     = true;
    private static final int _keysize    = 1024; // 4096;
    private static HMACDHHandler oneAndOnly = null; 
    
    private Hashtable establishedParties = new Hashtable();
    private Hashtable pendingParties = new Hashtable();
    private KeyPairGenerator _kpgen;
    private KeyAgreement _kagree;
    private SecureRandom _r;
    private Signature _signEngine;
    private String _keyfile;
    private QSPublicKey _sigPub;
    private QSPrivateKey _sigPri;
    private Mac _hmac;
    private SecureHash _myGuid;
    private SinkIF _this_sink;
    private ssTimer _timer;
    private int _stsTime;
    private int _retryLimit;
    private int _dhlifespan;

	public  MacHandler _mhandler;
    public  PartyInfo AlfInfo = new PartyInfo();
    public  QSPublicKey AlfPub;
    public  QSPrivateKey AlfPri;
    public  String _alfkeyfile;
    public  PartyInfo BigFootInfo = new PartyInfo();
    public  QSPublicKey BigFootPub;
    public  QSPrivateKey BigFootPri;
    public  String _bigfootkeyfile;
    public  int countdown = 1;
    private Classifier _classifier;

    public void handleEvent(QueueElementIF item)
        throws EventHandlerException {
   
        if (item instanceof MacHandler.HandshakeFailed) {
            MacHandler.HandshakeFailed announce = 
                (MacHandler.HandshakeFailed) item;
            System.out.println("                " + 
                               "()()()()()()()()()()()()()()()()()()()()()");
            System.out.println("                " +
                               "HandshakeFailed: " + announce.guid);
            System.out.println("                " + 
                               "()()()()()()()()()()()()()()()()()()()()()");
        }
        if (item instanceof MacHandler.KeyEstablished) {
            MacHandler.KeyEstablished announce = 
                (MacHandler.KeyEstablished) item;
            System.out.println("                                         " +
                               "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX");
            System.out.println("                                         " +
                               "Key Established: " + announce.guid);
            System.out.println("                                         " +
                               "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX");
            if (countdown >= 4) System.exit(0);
            //if (countdown >= 5) System.exit(0);
        } 
        if (item instanceof DHMessage) {
         try {
            byte[] seed;
            DHMessage out;
            DHMessage msg = (DHMessage) item;
            System.out.println("[~.~GOT EVENT--stagetester~.~]");
            msg.print();
            countdown +=1;
            switch (countdown) {
                case 2: 
                    // Confirm signature and Compute secret, Send message (3)
                    System.out.println("[=> stagetester:sending out FINALIZE]");

                    // Confirm signature ... NO not for testing
                    // copy sig into AlfInfo
                    AlfInfo.sigpub = msg.sigkey;

                    // Compute secret
                    _kagree.init(AlfInfo.dhpri);
                        // JUMP InvalidKeyException
                    _kagree.doPhase( msg.QSkey.key(), true);
                        // JUMP InvalidKeyException
                    seed = _kagree.generateSecret();
                    AlfInfo.secret = new RawSecretKey("DH seed", seed);

                    // ought to checkDHVerify
                    if (!checkDHVerify( msg.user_data, msg.mac, msg, AlfInfo)){
                            //JUMP InvalidKeyException
                            //JUMP SignatureException
                            //JUMP TypeTable.NoSuchTypeCode
                        // ignore (TODO write error to log file)
                        System.out.println("StageTester: " +
                                                     "dhverify failed.");
                        AlfInfo.print();
                        return;
                    }

                    // Send FINALIZE
                    out = new DHMessage(_myGuid, // sort of right
                                                  null,
                                                  (QSRsaPublicKey)_sigPub,
                                                  new QSHMACMD5(),
                                                  null, null, // makeDHVerify later
                                                  DHMessage.FINALIZE);
                    out.inbound = true; // true b/c of no tapestry

                    // compute verification info and place into DHVerify
                    // MAC(Signature(Hash(both DH keys and Mac type)))
                    makeDHVerify( new QSDHPublicKey(AlfInfo.dhpub),
                                  msg.QSkey,
                                  AlfInfo.secret,
                                  out);
                    // JUMP SignatureException
                    System.out.println("----StageTester alfinfo");
                    AlfInfo.print(); 

                    // sendout
                    _classifier.dispatch(out);
                    out = null;
                    // JUMP SinkException

                    //--------------------end of "I AM ALF" TEST
                    
                    //---------------------begin "I AM BIGFOOT" TEST 
                    // switch the keys to BigFoot keys.
                    _keyfile = _bigfootkeyfile;
                    // reinit
                    initSigEngine();
                        // JUMP NoSuchProviderException
                        // JUMP NoSuchAlgorithmException
                        // JUMP QSException

                    //get HMACDHHandler a TellSelf to get started... 
					_mhandler.establish(_myGuid);

                    break;
                case 3: 
                    // GOOOOOO BIGFOOT!!!
                    System.out.println("NEW TEST NEW TEST NEW TEST NEW TEST " +
                                       "NEW TEST NEW TEST NEW TEST NEW TEST " +
                                       "NEW TEST NEW TEST NEW TEST NEW TEST ");
                    /** 
                      // or let's try alf.
                      countdown = 1;
                      runTest();
                    */ 

                    // assuming we got an INIT message.
                    // Compute Secret 
                    _kagree.init(BigFootInfo.dhpri);
                        // JUMP InvalidKeyException
                    _kagree.doPhase( msg.QSkey.key(), true);
                        // JUMP InvalidKeyException
                    seed = _kagree.generateSecret();
                    BigFootInfo.secret = new RawSecretKey("DH seed", seed);

                    // Send Message RESPOND
                     out = new DHMessage( _myGuid, //strangely okay
                                                   BigFootInfo.dhpub,
                                                   (QSRsaPublicKey)_sigPub,
                                                   new QSHMACMD5(),
                                                   null, null, // makeDHVerify later
                                                   DHMessage.RESPOND);
                    out.inbound = true; // true b/c of no tapestry
                    makeDHVerify( new QSDHPublicKey(BigFootInfo.dhpub),
                                  msg.QSkey,
                                  BigFootInfo.secret,
                                  out);
                    // JUMP SignatureException
                    System.out.println("StageTester: sending out RESPOND");
                    _classifier.dispatch(out);
                    out = null;

                    break;
                case 4:
                    /**
                    System.out.println("STAGE TESTER REQUEST ESTABLISH");
					_mhandler.establish(_myGuid);
                    System.out.println("STAGE TESTER REQUEST ESTABLISH");
					_mhandler.establish(_myGuid);
                    BufferedReader console = new BufferedReader(
                        new InputStreamReader(System.in));
                    console.readLine();           
                    */ 
                    System.exit(0);
                    break;
            }
         //} catch (IOException e) {
         } catch (InvalidKeyException e) {
            Carp.die(e);
         } catch (SignatureException e) {
            Carp.die(e);
         } catch (TypeTable.NoSuchTypeCode e) {
            Carp.die(e);
         } catch (SinkException e) {
            Carp.die(e);
         } catch (NoSuchProviderException e) {
            Carp.die(e);
         } catch (NoSuchAlgorithmException e) {
            Carp.die(e);
         } catch (QSException e) {
            Carp.die(e);
         }
        }
    }

    public void handleEvents(QueueElementIF item[])
        throws EventHandlerException {
        for (int i = 0; i < item.length; i += 1)
            handleEvent(item[i]);
    }

    private boolean checkMAC( byte[] src, byte[] MAC, SecretKey macSecret) {
        try {
            _hmac.init(macSecret);
            return ByteUtils.equals(MAC, _hmac.doFinal(src));
        } catch (InvalidKeyException e) {
            Carp.die("HMACDHHandler: could not init hmac engine.");
            return false;
        }
    }

    private static byte [] data_to_check (QuickSerializable user_data) {
	byte [] result = new byte [4+ByteUtils.qs_length (user_data)];
	int [] offset = {0};
	ByteUtils.intToBytes (user_data.type_code (), result, offset);
	user_data.to_bytes (result, offset);
	return result;
    }

    private boolean checkDHVerify( QuickSerializable user_data,
				   QSByteArray mac, 
                                   DHMessage dhmsg, 
                                   PartyInfo info) 
     throws InvalidKeyException, SignatureException, TypeTable.NoSuchTypeCode {
        boolean allzgood;
        SignedQS s = null;
        STSVerify recreatedSTS;
        byte[] recreated, extracted;

        // check MAC
        allzgood = checkMAC(data_to_check (user_data), 
		mac.bytes (), info.secret);

        if (allzgood) {
            // extract signature
            s = (SignedQS) user_data;
        
            // verify Signature
            // next line: workaround cryptix bug
            try {reInitSigEngine();} catch (Exception e) {Carp.die(e);}
            allzgood = s.verify(info.sigpub.key(), _signEngine);
        }
        System.out.println("checkDH: sig. allzgood = " + allzgood);

 
        // check the HASH
        if (allzgood) {
            // recreate STS (as if they would have done, i.e.
            // "my Public + other Public + MACType"
            // becomes: 
            // "other Public + my Public + MACType"
            recreatedSTS = new STSVerify(dhmsg.QSkey, 
                                         new QSDHPublicKey(info.dhpub), 
                                         Types.MACTYPE_HMACMD5);

            // verify the HASH is equivalent
            allzgood = ((SHA1Hash)s.user_data()).verify(recreatedSTS);
        }
        System.out.println("checkDH: sig & hash allzgood = " + allzgood);

        return allzgood;
            
    }

    private void makeDHVerify( QSDHPublicKey myPu, 
                             QSDHPublicKey otherPu, 
                             SecretKey macSecret,
                             DHMessage destination ) 
     throws SignatureException, InvalidKeyException {
                            
        // (1) create STSVerify info my Public + other Public + MACType
        STSVerify sts = new STSVerify( myPu, otherPu, Types.MACTYPE_HMACMD5);
        // (2) Hash the info
        SecureHash digested = new SHA1Hash( sts);
        // (3) Sign the Hash
        // next line: workaround cryptix bug
        try {reInitSigEngine();} catch (Exception e) {Carp.die(e);}
        SignedQS sig = new SignedQS( digested, 
                                     new QSSha1RsaSignature(),
                                     _sigPri.key(),
                                     _r,
                                     _signEngine);
        // (4) MAC the Signature (the SHA1Hash)
        _hmac.reset();
        _hmac.init(macSecret);
	QSByteArray mac = new QSByteArray( _hmac.doFinal(data_to_check( sig)));
        // (5) insert into destination
        destination.user_data = sig;
        destination.mac = mac;
    }

    private void makeNewDHKeys( PartyInfo info) {
        // TODO in the future, we could create a keygenerator
        // with functional access that we could simple grab
        // a ready made key. (might save some time, this can take up to 8 secs)
        KeyPair pair = _kpgen.generateKeyPair();
        info.dhpub = (DHPublicKey) pair.getPublic();
        info.dhpri = (DHPrivateKey) pair.getPrivate();
    }

    public void init(ConfigDataIF config) throws Exception {

        System.err.println("Stage Tester initializing.------------");
        Types.registerAllTypes();

	_classifier = Classifier.getClassifier (
		new NodeId (config.getString ("node_id")));

	String stageName = config.getStage().getName();
	try 
	{
	  if(config.getBoolean("Profile"))
	  {
	    config.getManager().getProfiler().add("ostore.mac.StageTester", this);
	    SinkIF sink =
	      config.getManager().getStage(stageName).getSink();
	    _this_sink = new SinkProxy(sink,
				       config.getManager(),
				       config.getStage().getWrapper());
	  }
	  else
	    _this_sink =
	      config.getManager().getStage(stageName).getSink();
	} 
	catch (NoSuchStageException e) 
	{
	  Carp.die (stageName + ".init: could not get own sink.");
	}

        _timer = new ssTimer();

        // sts timeout length (milliseconds)
        _stsTime = config.getInt("timeoutlength");

        // dh key lifespan (milliseconds)
        _dhlifespan = config.getInt("dhlifespan");
    
        // get sts retry limit
        _retryLimit = config.getInt("sessionretries");

        // get my signature key filename
        _keyfile = config.getString("sigkeyfile");
        _alfkeyfile = config.getString("alfkey");
        _bigfootkeyfile = config.getString("bigfootkey");

        // subscribe ourselves to certain types of messages
        // DHMessages (inbound)
        Filter filter = new Filter();
        if (! filter.requireType (Class.forName
            ("ostore.mac.DHMessage")))
        Carp.die ("StageTester: could not require type.");
        // making things backwards because we are 
        // inhabiting the same sandstorm instance as
        // the HMACDHHandler we are testing
        if (! filter.requireValue ("inbound", new Boolean (false)))
            Carp.die ("StageTester: could not require inbound.");
        _classifier.subscribe( filter, _this_sink);

        filter = new Filter();
        if (! filter.requireType (Class.forName
            ("ostore.mac.MacHandler$HandshakeFailed")))
        Carp.die ("StageTester: could not require type.");
        _classifier.subscribe( filter, _this_sink);

        filter = new Filter();
        if (! filter.requireType (Class.forName
            ("ostore.mac.MacHandler$KeyEstablished")))
        Carp.die ("StageTester: could not require type.");
        _classifier.subscribe( filter, _this_sink);

        try {
            initProviders();
            initKeyGenEngines();
            initSigEngine();
            initMacEngine();
            runTest();
        } catch (Exception e) {
            Carp.die(e);
        }
    }

    void initProviders()
     throws NoSuchProviderException,
            NoSuchAlgorithmException {    
	System.out.println("Adding Provider: CryptixJCE");
        Provider cryptixJCE = new cryptix.jce.provider.CryptixCrypto();
        Security.addProvider(cryptixJCE);
        //cryptixJCE.list(System.out);

        System.out.println("Adding Provider: CryptixRandom");
        Provider cryptixRandom = new cryptix.jce.provider.CryptixRandom();
        Security.addProvider(cryptixRandom);
        //cryptixRandom.list(System.out);

        // Initialize

          // Initialize SecureRandom (Random number generator)
          boolean available = DevRandom.isAvailable();
          System.err.println("DevRandom.isAvailable = " + available);
          // Note that we are using DevRandom from CryptixRandom
          // which pulls their numbers from /usr/random
          // which, by the way, is butt fast.
          _r = SecureRandom.getInstance("DevRandom","CryptixRandom");
    }

    void initKeyGenEngines() 
     throws NoSuchAlgorithmException,
            NoSuchProviderException {
        _kpgen = KeyPairGenerator.getInstance("DH","CryptixCrypto");
        _kpgen.initialize( _keysize, _r);
        _kagree = KeyAgreement.getInstance("DH", "CryptixCrypto");
    }

    /**
     * initializes the signature engine, grabs a precomputed signature key pair
     * and calculates a guid from that pair.
     */
    void initSigEngine() 
     throws NoSuchProviderException,
            NoSuchAlgorithmException,
            InvalidKeyException,
            QSException {
        _signEngine = Signature.getInstance("SHA-1/RSA/PKCS#1", "CryptixCrypto");
        //TODO should we be grabbing this from some global config for
        // this OceanStore instance? we ought to have a EncryptedQSIO

        // Grab keys from _keyfile.pub and _keyfile.priv
        try {
            _sigPub = (QSPublicKey) QSIO.read(_keyfile + ".pub");
            _sigPri = (QSPrivateKey) QSIO.read(_keyfile + ".priv");
        } catch (IOException e) {
            Carp.die ("HMACDHHandler: IO Exception reading " +
                      _keyfile + ".pub or " + _keyfile + ".priv");
        }
        //_signEngine.initSign(_sigPri.key(), _r); UNNECESSARY

        // get my guid
        _myGuid = new SHA1Hash (_sigPub);
    }
    
    void reInitSigEngine()
     throws NoSuchProviderException,
            NoSuchAlgorithmException,
            InvalidKeyException {
        _signEngine = Signature.getInstance("SHA-1/RSA/PKCS#1", "CryptixCrypto");
    }

    void initMacEngine() 
     throws NoSuchAlgorithmException,
            NoSuchProviderException {
        // Initalize the Mac Engine
        _hmac = Mac.getInstance("HMAC-MD5", "CryptixCrypto");
    }

    void runTest() {
        // Ought to be a simple framework 1st.
        //--------------------begin "I AM ALF" TEST
        System.out.println("STARTING 'I AM ALF' TEST ------------------------");

        // FIND HMACINSTANCE note: WE AREN'T using this
        System.out.print("[**] getting instance of HMAC...");
        MacHandler mhandler = HMACDHHandler.getInstance();
		_mhandler = mhandler;
        if (mhandler != null) {
            System.out.println(mhandler.idString());
        } else {
            System.out.println("ERROR. No instance retrieved.");
        }


        // create DHKeys for Alf 
        System.out.print("(1) Alf: create a set of dh keys of size " + 
                         _keysize + " ...");
        makeNewDHKeys(AlfInfo);
        System.out.println("done.");

        // create DHKeys for Bigfoot 
        System.out.print("(2) Bigfoot: create a set of dh keys of size " + 
                           _keysize + " ...");
        makeNewDHKeys(BigFootInfo);
        System.out.println("done.");

        // get signature keys from .pri and .pub external files
        // we did this already and got: _sigPub and _sigPri

        // MESSAGE TESTING
        System.out.print("[A] Sending INIT to HMACDHHandler...");
        DHMessage out = new DHMessage(_myGuid,// its okay like this b/c 
										      // we don't really use tapestry 
                                      AlfInfo.dhpub,
                                      null,
                                      new QSHMACMD5(),
                                      null, null,
                                      DHMessage.INITIALIZE);
        out.inbound = true;
        // send it the message

        _classifier.dispatch_later(out, 1004);
        System.out.println("SENT.");

        // THEN THE JOB GOES TO THE EVENT REACTION... 
    }

    public void destroy() throws Exception {
        //TODO
        // free timer?
    }

    //  methods for ProfilableIF
    /* Specified by mdw.sandStorm.api.ProfilableIF */
    public int profileSize()
    {
	return _this_sink.size();
    }
    //  end methods for ProfilableIF

}
