/**
 * Key with a type and provider.
 *
 * @author     Sean C. Rhea
 * @version    $Id: GenerateRSAKeyPair.java,v 1.5 2001/10/26 23:51:29 srhea Exp $
 */

package ostore.security;
import ostore.util.*;
import java.security.*;
import java.io.*;

public class GenerateRSAKeyPair {

    public static KeyPair genkeypair (int size, boolean debug) {
	KeyPair keypair = null;
	try {
	    Provider cryptix = new cryptix.jce.provider.CryptixCrypto ();
	    Security.addProvider (cryptix);

	    Provider cryptixRandom = new cryptix.jce.provider.CryptixRandom ();
	    Security.addProvider (cryptixRandom);
	    SecureRandom secure_random = 
		SecureRandom.getInstance ("DevRandom", "CryptixRandom");
	    KeyPairGenerator kpgen = 
		KeyPairGenerator.getInstance ("RSA", "CryptixCrypto");
	    kpgen.initialize (size, secure_random);
	    keypair = kpgen.generateKeyPair ();
	}
	catch (Exception e) {
	    if (debug) Carp.out ("Caught exception " + e 
		    + " while trying to generate a key pair.");
	    return null;
	}
	return keypair;
    }

    public static QSPublicKey qs_pkey (KeyPair kp) {
	return new QSRsaPublicKey (kp.getPublic ());
    }

    public static QSPrivateKey qs_skey (KeyPair kp) {
	return new QSRsaPrivateKey (kp.getPrivate ());
    }

    public static void main (String [] argv) {

	if (argv.length < 2) {
	    System.err.println ("usage: genkey <prefix> <size>");
	    System.exit (1);
	}

	int which = 0;
	String prefix = argv[which++];
	int keysize = Integer.parseInt (argv[which++]);

	KeyPair keypair = genkeypair (keysize, true);
	if (keypair == null) {
	    System.exit (1);
	}

	QSPublicKey  pub  = qs_pkey (keypair);
	QSPrivateKey priv = qs_skey (keypair);

	try {
	    QSIO.write (pub, prefix + ".pub");
	}
	catch (IOException e) {
	    System.err.println ("Could not write " + prefix + ".pub");
	    System.exit (1);
	}

	try {
	    QSIO.write (priv, prefix + ".priv");
	}
	catch (IOException e) {
	    System.err.println ("Could not write " + prefix + ".priv");
	    System.exit (1);
	}
    }
}
