/**
 * QSDHPublicKey.
 *
 * @author      Gifford Cheung
 * @version     $Id: QSDHPublicKey.java,v 1.2 2002/03/29 01:59:58 srhea Exp $
 */

package ostore.security;

import java.math.BigInteger;
import java.security.*;
import java.util.*;
import javax.crypto.interfaces.*;
import javax.crypto.spec.*;

import ostore.util.*;

/**
 * QuickSerializable Diffie-Hellman public key. Based on the same
 * the Cryptix JCE DHPublicCryptix v1.1 and QSPublicKey
 *
 * @version $Revision 1.0$
 * @author Gifford Cheung 
 */
public class QSDHPublicKey implements QSPublicKey {

	public QSDHPublicKey (DHPublicKey key) {
		Assert.assert( key != null, "QSDHPublicKey: key must be non-null.");
		_key = new MyRawDHPublicKey(key.getY(), key.getParams());
	}
	public QSDHPublicKey(byte[] data, int[] offset) throws QSException {
		BigInteger y,p,g;
		int l;
		/*byte[] temp;
		int tlen;*/
		// y
		y = ByteUtils.bytesToBigInteger (data, offset); /*
		tlen = ByteUtils.bytesToInt(data, offset);	
		temp = new byte[tlen];
		offset[0] += ByteUtils.memcpy( temp, 0, data, offset[0], tlen);
		y = new BigInteger(temp);*/
		// params (p, g, and l)
		// p
		p = ByteUtils.bytesToBigInteger (data, offset); /*
		tlen = ByteUtils.bytesToInt(data, offset);	
		temp = new byte[tlen];
		offset[0] += ByteUtils.memcpy( temp, 0, data, offset[0], tlen);
		p = new BigInteger(temp);*/
		// g
		g = ByteUtils.bytesToBigInteger (data, offset); /*
		tlen = ByteUtils.bytesToInt(data, offset);	
		temp = new byte[tlen];
		offset[0] += ByteUtils.memcpy( temp, 0, data, offset[0], tlen);
		g = new BigInteger(temp);*/
		// l 
		l = ByteUtils.bytesToInt (data, offset);
		_key = new MyRawDHPublicKey( y, new DHParameterSpec( p, g, l));
	}

	public void to_bytes (byte[] data, int[] offset) {
		// y
		ByteUtils.bigIntegerToBytes (_key.getY(), data, offset); /*
		byte[] ybytes = _key.getY().toByteArray();
		ByteUtils.intToBytes(ybytes.length, data, offset);
		offset [0] += ByteUtils.memcpy(data, offset[0], 
					       ybytes, 0, ybytes.length);*/
		// params (p, g, and l)
		// p
		ByteUtils.bigIntegerToBytes (
			_key.getParams().getP(), data, offset); /*
		byte[] pbytes = _key.getParams().getP().toByteArray();
		ByteUtils.intToBytes(pbytes.length, data, offset);
		offset [0] += ByteUtils.memcpy(data, offset[0], 
					       pbytes, 0, pbytes.length);*/
		// g
		ByteUtils.bigIntegerToBytes (
			_key.getParams().getG(), data, offset); /*
		byte[] gbytes = _key.getParams().getG().toByteArray();
		ByteUtils.intToBytes(gbytes.length, data, offset);
		offset [0] += ByteUtils.memcpy(data, offset[0], 
					       gbytes, 0, gbytes.length);*/
		// l 
		ByteUtils.intToBytes(_key.getParams().getL(), data, offset);
	}

	public void add_to_digest( SecureHashBuffer buffer ) {
	    buffer.add (_key.getY());
	    buffer.add (_key.getParams().getP());
	    buffer.add (_key.getParams().getG());
	    buffer.add (_key.getParams().getL());
	}

	public int type_code() { return Types.SECURITY_QS_DH_PUBLIC_KEY; }
	
	public PublicKey key() { return _key; }

	public boolean equals( Object obj) {
		boolean e;
		if (obj == null) 
			e = false;
		else if (obj == this) 
			e = true;
		else if (!(obj instanceof QSDHPublicKey))
			e = false;
		else {
			QSDHPublicKey otherkey = (QSDHPublicKey) obj;
			DHPublicKey key = (DHPublicKey) otherkey.key();
			// y 
			boolean y =
			_key.getY().equals( key.getY());
			// p
			boolean p =
			_key.getParams().getP().equals( key.getParams().getP());
			// g
			boolean g =
			_key.getParams().getG().equals( key.getParams().getG());
			// l
			boolean l =
			_key.getParams().getL() == key.getParams().getL();
			e = (y && p && g && l);
		}
		return e;
			
	}

	DHPublicKey _key;

	private class MyRawDHPublicKey implements DHPublicKey {

		private final BigInteger y;
		private final DHParameterSpec params;

		public MyRawDHPublicKey(BigInteger y, DHParameterSpec params) {
			this.y = y;
			this.params = params;
		}

		public BigInteger getY() { return this.y; }
		public DHParameterSpec getParams() { return this.params; }
		public String getAlgorithm() { return "DH"; }
		public String getFormat() { return "RAW"; }
		public byte[] getEncoded() {
			//TODO uh... not used? 
			Carp.die("Not implemented");
			return new byte[0];
		}
	}
}
	
