/**
 * QSRsaPublicKey.
 *
 * @author     Sean C. Rhea
 * @version    $Id: QSRsaPublicKey.java,v 1.5 2002/03/29 01:59:58 srhea Exp $
 */

package ostore.security;
import cryptix.jce.provider.rsa.RSAPublicKeyCryptix;
import java.math.BigInteger;
import java.security.PublicKey;
import ostore.util.Assert;
import ostore.util.ByteUtils;
import ostore.util.QSException;
import ostore.util.Types;
import ostore.util.SecureHashBuffer;
import ostore.util.SHA1Hash;

/**
 * A subtype of <code>QSPublicKey</code> for RSA public keys.
 * 
 * @author   Sean C. Rhea
 * @version  $Id: QSRsaPublicKey.java,v 1.5 2002/03/29 01:59:58 srhea Exp $
 */
public class QSRsaPublicKey implements QSPublicKey {

    private static final String class_tag = "QSRsaPublicKey";

    public String toString () {
	return "(QSRsaPublicKey " + (new SHA1Hash (this)) + ")";
    }

    public QSRsaPublicKey (PublicKey key) {
	final String tag = class_tag + ".QSRsaPublicKey";
	Assert.assert (key != null, tag + ": key must be non-null.");
	Assert.assert (key instanceof RSAPublicKeyCryptix, 
		tag + ": key must be of type RSAPublicKeyCryptix, " +
		"not of type " + key.getClass ().getName () + ".");
	_key = (RSAPublicKeyCryptix) key;
    }

    public QSRsaPublicKey (byte [] data, int [] offset) throws QSException {
	BigInteger e = ByteUtils.bytesToBigInteger (data, offset);
	BigInteger n = ByteUtils.bytesToBigInteger (data, offset);
	_key = new RSAPublicKeyCryptix (n, e);

	/*
	ByteArrayInputStream is = 
	    new ByteArrayInputStream (data, offset [0], length);
	try {
	    BigInteger e = BI.fromStream(is);
	    BigInteger n = BI.fromStream(is);
	    _key = new RSAPublicKeyCryptix (n, e);
	}
	catch (IOException e) {
	    throw new QSException ("Bad encoding.");
	}
	offset [0] += length;
	*/
    }

    public void to_bytes (byte [] data, int [] offset) {
	ByteUtils.bigIntegerToBytes (_key.getPublicExponent (), data, offset);
	ByteUtils.bigIntegerToBytes (_key.getModulus (), data, offset);
	/*
	ByteArrayOutputStream os = new ByteArrayOutputStream ();
	try {
	    BI.toStream (e, os);
	    BI.toStream (n, os);
	}
	catch (IOException e) {
	    Assert.assert (false, tag + ": could not encode BigInteger.");
	}
	byte [] tmp = os.toByteArray ();
	ByteUtils.intToBytes (tmp.length, data, offset);
	offset [0] += ByteUtils.memcpy (data, offset [0], tmp, 0, tmp.length);
	*/
    }

    public void add_to_digest( SecureHashBuffer buffer ) {
	buffer.add (_key.getPublicExponent ());
	buffer.add (_key.getModulus ());
    }

    public int type_code () { return Types.SECURITY_QS_RSA_PUBLIC_KEY; }

    public PublicKey key () {
	return _key;
    }

    public boolean equals (Object other) {
	if (other == null)
	    return false;
	if (other == this)
	    return true;
	if (! (other instanceof QSRsaPublicKey))
	    return false;
	QSRsaPublicKey rhs = (QSRsaPublicKey) other;
	return _key.getModulus  ().equals (rhs._key.getModulus  ()) &&
	       _key.getPublicExponent ().equals (rhs._key.getPublicExponent ());
    }

    protected RSAPublicKeyCryptix _key;
}

