/**
 * SignedQSArray.java
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.security;
import ostore.util.*;
import java.security.*;

/**
 * Documentation, what's that?
 *
 * @author   Sean C. Rhea
 * @version  $Id: SignedQSArray.java,v 1.4 2002/04/06 22:52:43 eaton Exp $
 */
public class SignedQSArray /* implements QuickSerializable */ {

    public static class Individual extends SignedQS {

	public Individual (QuickSerializable user_data, 
		SignedQS signed_object_hashes, int number) {
	    _user_data = user_data;
	    _signed_object_hashes = signed_object_hashes;
	    _number = number;
	}
    
	public Individual (byte [] data, int [] offset) throws QSException {
            long user_data_type = ByteUtils.bytesToLong (data, offset);
	    _user_data = TypeTable.new_object (user_data_type, data, offset);
	    _signed_object_hashes = new SignedQS (data, offset);
	    _number = ByteUtils.bytesToInt (data, offset);
	}

	public QuickSerializable user_data () { return _user_data; }

	public QSSignature qs_signature () throws TypeTable.NoSuchTypeCode {
	    return _signed_object_hashes.qs_signature ();
	}

	public void to_bytes (byte [] data, int [] offset) {
	    ByteUtils.longToBytes (TypeTable.type_code(_user_data),
                                   data, offset);
	    _user_data.to_bytes (data, offset);
	    _signed_object_hashes.to_bytes (data, offset);
	    ByteUtils.intToBytes (_number, data, offset);
	}

	/* Specified by ostore.util.Digestible */
	public void add_to_digest( SecureHashBuffer buffer )
	{
	    buffer.add( _user_data.type_code() );
            //  TODO: Change type and remove cast.
	    buffer.add( (Digestible) _user_data );
	    buffer.add( _signed_object_hashes );
	    buffer.add( _number );
	}

	public int type_code () {
            throw new NoSuchMethodError();
	    //return Types.SECURITY_SIGNED_QS_ARRAY_IND;
	}

	public boolean verify (PublicKey pubkey, Signature engine) 
	throws InvalidKeyException, SignatureException, 
	TypeTable.NoSuchTypeCode {
	    boolean result = false;
	    try {
		if (_signed_object_hashes.verify (pubkey, engine)) {
		    SecureHash h = new SHA1Hash ((Digestible) _user_data);
		    QSArray hash_array = (QSArray)
			_signed_object_hashes.user_data ();
		    SecureHash g = (SecureHash) hash_array.get (_number);
		    result = h.equals (g);
		}
	    }
	    catch (InvalidKeyException e) {
		throw e;
	    }
	    catch (SignatureException e) {
		throw e;
	    }
	    catch (TypeTable.NoSuchTypeCode e) {
		throw e;
	    }
	    catch (Exception e) {
		return false;
	    }
	    return result;
	}

	private SignedQS _signed_object_hashes;
	private int _number;
    }

    public SignedQSArray (QuickSerializable [] user_data,
	    QSSignature qs_signature,
	    PrivateKey privkey,
	    SecureRandom random,
	    Signature engine) throws InvalidKeyException, SignatureException {

	QuickSerializable [] hash_array = 
	    new QuickSerializable [user_data.length];
	for (int i = 0; i < hash_array.length; ++i)
	    hash_array [i] = new SHA1Hash ((Digestible) user_data [i]);
	QSArray qs_hash_array = new QSArray (hash_array);
	_signed_object_hashes = new SignedQS (qs_hash_array, qs_signature, 
		privkey, random, engine);
	
	_signed_objects = new SignedQS [hash_array.length];
	for (int i = 0; i < hash_array.length; ++i)
	    _signed_objects [i] = 
		new Individual (user_data [i], _signed_object_hashes, i);
    }

    public SignedQS signed_object (int number) {
	return _signed_objects [number];
    }

    /*
    public SignedQSArray (byte [] data, int [] offset) throws QSException {
    }

    public void to_bytes (byte [] data, int [] offset) {
    }

    public int type_code () {
	return Types.SECURITY_SIGNED_QS_ARRAY;
    }
    */

    private SignedQS [] _signed_objects; // are Individuals
    private SignedQS _signed_object_hashes; // user_data () isa SecureHash []

}

