/** Manages all of the logic and pointers to which nodes go where,
 *  etc.
 * TODO : remove doesn't work well.<br>
 * TODO : test code is not current ( but close, just some function interfaces that are different)<br>
 *
 * @version 1.0
 * @author Scott Honey */

package ostore.tapestry;

import java.util.*;
import java.net.*;
import ostore.util.*;

public class NeighborTable { 

    //temp debug output.
    boolean SLADEBUG=false;//sort link array
    boolean BLDEBUG=false; //blacklisting
    boolean GNNDEBUG=false;//getNextNode

    /////////////////////////////////////////////////
    //Variables
    
    //not really constants, but treat as such.  (in case multiple
    //instances of tapestries running or something.) Originally was
    //static.
    private int LENGTH;
    private int BASE;
    private int BACKUPS;
 
    
    /**All of the pointers that are currently being used. */
    private Link m_lookupTable[][][];
    
    
    /**Looks up Links/Nodes that are currently IN the lookupTable
       keys are guids, objects are Links
     */
    private Hashtable m_linkHash;

    /**Looks up Links/Nodes that link to us.
       keys are guids, objects are Links
     */
    private Hashtable m_backpointers;
    
    /** Big'o list of untrustable nodes.
	keys are guids, objects are guids.
     */
    private Hashtable m_blacklist;     


    private SHA1Hash m_selfGuid;
    private int []   m_selfDecode;
    private Link     m_selfLink;
    private NodeId   m_selfNode;


    /* following structures not necessary -- taken care of by linkLookup

    private Hashtable m_backpointers;
    private Hashtable m_nodeToLink;
    */



    /////////////////////////////////////////////////
    // Constructors

    
    public NeighborTable(int length, int base, int backups, NodeId selfNode){
	
	//TODO some sort of error checking to make sure that the
	//lengths are the appropriate powers of two.
	//These must be place before calls to decodeGuid
	LENGTH=length;
	BASE=base;
	BACKUPS=backups;
	
	
	m_lookupTable = new Link[length][base][backups];
	
	m_linkHash = new Hashtable(length*base*backups);
	m_blacklist = new Hashtable();
	m_backpointers = new Hashtable();

	m_selfNode = selfNode;
	m_selfGuid = new SHA1Hash(m_selfNode);
	m_selfDecode= decodeGuid(m_selfGuid);
	m_selfLink = new Link(m_selfGuid,m_selfNode,0);
	m_selfLink.setLocal();
	
	add(m_selfLink); //looking good.  
	
    }
    
    
    


    ////////////////////////////////////////////////////////////
    // public functions

    /** Public interface for adding a link into the table.
	@param newLink the link you want to add.
	DONE
    */
    public void add(Link newLink){

	//if already is part of us, then we don't need to re-add it.
	if (m_linkHash.containsKey(newLink.getGuid())){
	    return;
	}
		    
	//if blacklisted, then we don't want it.  WIP TODO the getGuid is a hack.
	if (m_blacklist.containsKey(newLink.getGuid())){
	    return;
	}
	
	
	//WIP TODO shold add it only on success, right now add all the time.
	
	m_linkHash.put(newLink.getGuid(),newLink);
	
	//if we backpoint to it, then we should probably combine the
	//Link objects.  TODO.
	
	
	//find out the deepest it matches our table.
	SHA1Hash guid = newLink.getGuid();
	int[] translation = decodeGuid(guid);
	int matchIndex = findDifference(translation, m_selfDecode);// + 1; //add one for first MATCH.
	
	
	//in case machine is adding itself, don't want a bad array access.
	if (matchIndex==-1) {
	    matchIndex++;
	}
	
	//try adding for every link that it matches.
	while (matchIndex<LENGTH){
	    if (SLADEBUG) System.out.println("SLA Match : "+matchIndex);
	    suggestLink(matchIndex,translation,newLink);
	    matchIndex++;
	}
    }
    




    /** call this when finally sending something down the wire --
        it'll do cool stuff (like update the last time something got sent to the node)*/
    public void sent(NodeId dest){
	Link nodeInfo;
	nodeInfo =(Link) m_linkHash.get(dest);

	if (nodeInfo==null) {
	    //something horrible wrong. TODO.
	}
	
	if (nodeInfo!=null) {
	    nodeInfo.sent();
	}
	
    }
    

    /** call this when getting something, it'll freshen the last time
	received (for heartbeats) (and may update mappings, as well? */
    
    public void received(NodeId from){
	Link nodeInfo;
	nodeInfo = (Link) m_backpointers.get(from);
	
	if (nodeInfo==null) {
	    //got something from a node that we didn't know pointed to us.  TODO. Perhaps add it?
	}
	
	if (nodeInfo!=null) {
	    nodeInfo.received();
	}
    }


    
    
    public void remove (NodeId removeNode){
	Link nodeInfo;
	
	//can no longer point to us, so remove from backpointers.
	nodeInfo = (Link) m_backpointers.get(new SHA1Hash(removeNode));
	if (nodeInfo!=null) {
	    m_backpointers.remove(removeNode);
	}
	
	//if we link to it in the table, set it as disintigrated.
	nodeInfo = (Link) m_linkHash.get(new SHA1Hash(removeNode));
	if (nodeInfo!=null) {
	    nodeInfo.setDisintigrated();
	}
	
	
		
	//TODO if this is the last acceptable match for the
	//destination, report error/start relinking?
    }
	
    
    public String pb(int[] ba){
	for (int i = 0; i<ba.length;i++){
	    System.out.print(Integer.toHexString((int)ba[i]));
	}
	
	return "";
	
    }

    public NodeId getExact(SHA1Hash dest){
	
	//if difference greater than 0 (i.e. the exact match will not
	//be in the neighbore table, return null; TODO -- this might not work.
	if (findDifference(m_selfDecode, decodeGuid(dest)) >0) {

	    return null;
	}
	return getNextNode(dest,0);
    }
    
    
    public NodeId getClosest(SHA1Hash dest){
	NodeId node;
	int base;
	
	base = findDifference(m_selfDecode,decodeGuid(dest));
	
	while (base < LENGTH) {
	    node=getNextNode(dest,base);
	    if (node!=null) {
		return node;;
	    }
	}
	
	return null;
    }



    /** Gets the next node to hop to, and reorders the table if the top node is bad.
     * @return Returns the next NodeId to hope to, or null if there is
     * nothing suitable.  */
    
    //    public NodeId getNextNode(NodeId dest, int hop) {
	
    public NodeId getNextNode(SHA1Hash dest, int hop) {
	//WIP	SHA1Hash WIPguid = newNode.getGuid();
	
	Link nodeInfo;
	
	int[] translation = decodeGuid(dest);	
	
	Link[] linkArray  = m_lookupTable[hop][translation[hop]];
	
	//first, check to make sure it should even be matching this far up.
	
	if (findDifference(translation,m_selfDecode)-hop>0){
	    //THIS is bad, should never get here. The message will not be routed correctly if continued.
	    if (GNNDEBUG) {
		System.out.print("~");
	    }
	    return null;
	}
	

	//ideal case where the first link listed is said to be up and running.
	nodeInfo = linkArray[0];
	if ((nodeInfo!=null) && (nodeInfo.isUp())){
	    if (GNNDEBUG){
		System.out.print("GNN first try worked! hop : " + hop + " ");
		System.out.print("Difference = " + findDifference(translation,m_selfDecode));
		System.out.println(" Routing on "+Integer.toHexString(translation[hop]));
		System.out.println("  local : " +pb(m_selfDecode));
		System.out.println("  dest  : " +pb(translation));
		System.out.println("  next  : " +pb(decodeGuid(nodeInfo.getGuid())));
		System.out.println();
	    }
	    return nodeInfo.getNodeId();
	}
	else {   //the first node is no good, so reorder the nodes and try again.
	    sortLinkArray(linkArray);
	    nodeInfo = linkArray[0];

	    if ((nodeInfo!=null) && (nodeInfo.isUp())){
		if (GNNDEBUG){
		    System.out.print("GNN second try worked! hop : "+ hop + "  ");
		    System.out.print("Difference = " + findDifference(translation,m_selfDecode));
		    System.out.println(" Routing on "+Integer.toHexString(translation[hop]));
		    System.out.println("  local : " +pb(m_selfDecode));
		    System.out.println("  dest  : " +pb(translation));
		    System.out.println("  next  : " +pb(decodeGuid(nodeInfo.getGuid())));
		    System.out.println();
		}
		return nodeInfo.getNodeId();
	    }   
	}
	
	//if gets to here, then there is no good link to follow.  Possibly throw some sort of alert?
	if (GNNDEBUG){
	    System.out.print("GNN nothing. hop : " + hop + "  ");
	    System.out.print("Difference = " + findDifference(translation,m_selfDecode));
	    System.out.println(" Routing on "+Integer.toHexString(translation[hop]));
	    System.out.println("  local : " +pb(m_selfDecode));
	    System.out.println("  dest  : " +pb(translation));
	    System.out.println();
	}
	return null;
	
    }
    
    


	
    


    /** note -- probably a slow implementation.
	@return returns an Enumeration of NodeIdeverybody who needs a heartbeat sent to them.
	@param delta how many milliseconds old does a link have to be before being marked old.
    */
    public Enumeration getNeedHeartbeats(long delta){
	
	Vector eligible = new Vector(m_backpointers.size());
	Enumeration allLinks = m_backpointers.elements();
	
	//to avoid making tons of calls to System.currentTimeMillis()
	//when the accuracy is not needed.
	long staleDate =  System.currentTimeMillis() - delta;
	Link nodeInfo;
	while (allLinks.hasMoreElements()){
	    nodeInfo = (Link)allLinks.nextElement();
	    if (nodeInfo.lastSent() < staleDate) {
		eligible.addElement(nodeInfo.getNodeId());
	    }
	}
	
	return eligible.elements();
	
    }
    
    
    
    /** Marks all nodes that haven't been heard from in a while as damaged or down.
	note -- probably a slow implementation, 
	but running on my slow (400mhz) box, it did 263 checks/millisecond
	@param delta how many milliseconds old does a link have to be before being marked old.
	TODO : Currently "down" happens after three periods worth of "damaged".  
	Make this more scientific.
	DONE
    */
    public void markOld(long delta){
	Enumeration allLinks = m_linkHash.elements();
	
	//to avoid making tons of calls to System.currentTimeMillis()
	//when the accuracy is not needed.
	long damageDate =  System.currentTimeMillis() - delta;
	long downDate   =  System.currentTimeMillis() - delta*3;
	Link nodeInfo;
	
	while (allLinks.hasMoreElements()){
	    nodeInfo = (Link)allLinks.nextElement();
	    if (nodeInfo.lastSent() < damageDate) {
		
		//if already damaged, check to see if should mark down.
		if ((nodeInfo.isDamaged()) && (nodeInfo.lastSent() <downDate)) {
		    nodeInfo.setDown();
		} else {

		    //if it was up, then mark it damaged.  It's
		    //important not to mark it damaged unless it was
		    //up, otherwise blacklisted nodes could be marked
		    //damaged instead of blacklisted.
		    if (nodeInfo.isUp()){
			nodeInfo.setDamaged();
		    }
		}
	    }//outer if
	    
	}//end while
	
    }
    
    
    
    
    
    
	


    /**Not to sure what this should do, but should probably trim some tables or just plain clean things up....  TODO.
    public void maintenance(){
    //prunes the tables? something?
    //checks if stuff is too old?
    }



    /** Marks a certain guid or nodes bearing a certain guid to be unusable.*/
    public void blacklist(NodeId theNode){
	Link nodeInfo;
	SHA1Hash guid = new SHA1Hash(theNode);
	if (BLDEBUG){
//	    System.out.println("BL set " + guid.fullString());
	}
	m_blacklist.put(guid,guid);

	nodeInfo = (Link) m_linkHash.get(guid);
	if (nodeInfo!=null) {
	    nodeInfo.setBlacklisted();
	    if (BLDEBUG){
	//	System.out.println("BL marked " + guid.fullString());
	    }
	}
	
	nodeInfo = (Link) m_backpointers.get(guid);
	if (nodeInfo!=null) {
	    nodeInfo.setBlacklisted();
	}
	
    }	
    /** UNmarks previously blacklisted nodes*/
    public void unBlackList(NodeId theNode){
	Link nodeInfo;
	SHA1Hash guid = new SHA1Hash(theNode);

	m_blacklist.remove(guid);
	
	nodeInfo = (Link) m_linkHash.get(guid);
	if (nodeInfo!=null) {
	    nodeInfo.setUnBlacklisted();
	}	
	nodeInfo = (Link) m_backpointers.get(guid);
	if (nodeInfo!=null) {
	    nodeInfo.setUnBlacklisted();
	}	
	
    }	
    


    ////////////////////////////////////////////////////////////
    // Table manipulation


    /** Compares the incoming link to the other links listed in the
        column/row in the lookupTable.
	Then, it takes the resulting best links and keeps them, throwing away the bad link.
	@param column the column to sort
	@param translation the expanded/decoded version of the SHA1Hash
	@param newLink the actual link.
	DONE
    */

    private void suggestLink(int column, int[] translation, Link newLink){

	//fancy way to declare a link of length BACKUPS + 1.
	Link[] linkArray = new Link[m_lookupTable[column][translation[column]].length + 1];
	
	//fill up the new array with the old values, appending the newLink.
	for (int x = 0; x < linkArray.length -1; x++){
	    linkArray[x] = m_lookupTable[column][translation[column]][x];
	}
	linkArray[linkArray.length - 1]=newLink;
	
	
	//sort it. 
	sortLinkArray(linkArray);
	
	
	//fill in the lookupTable with the best answers (with another way of saying BACKUPS)
	for (int x = 0; x < m_lookupTable[column][translation[column]].length; x++){
	    m_lookupTable[column][translation[column]][x] = linkArray[x];
	}

    }
    

    /**Sorts an array of links in place by the Link.compare method.
       @param linkArray the array to be sorted.
       DONE.
     */
    private void sortLinkArray(Link[] linkArray){
	
	if (SLADEBUG){
	    for (int a = 0; a < linkArray.length; a++){
		if (linkArray[a]!=null) System.out.println("SLA_START : "+a+" " +linkArray[a].getGuid().fullString()+" Status: " + linkArray[a].getStatus() + " latency: "+linkArray[a].getLatency());
	    }
	}  
	

	Link temp;
	int  i;
	boolean notDone;
	
	do {
	    notDone = false;               
	    for (i = 0;  i < linkArray.length -1;  i++) {
		if (Link.compare(linkArray[i],linkArray[i+1]) >0) { 
		    temp = linkArray[i];
		    linkArray[i] = linkArray[i+1];
		    linkArray[i+1] = temp;
		    notDone = true;
		}
	    }
	} while (notDone);          // if no swaps, then the list is sorted.
	
	if (SLADEBUG){
	    for (int a = 0; a <linkArray.length; a++){
	        if (linkArray[a]!=null) System.out.println("SLA_FINISH : "+a+" "+linkArray[a].getGuid().fullString()+" Status: " + linkArray[a].getStatus() + " latency: "+linkArray[a].getLatency()); 
		
	    }
	 }
	
    }  
    

    
    
        


    /////////////////////////////////////////////////////////////
    // Dynamic insertion stuff.	

    public Link[][] getColumn(int colNo){
	return null;
    }
    
    public void addColumn(int colNo, Link[][] column){
	
    }





    ////////////////////////////////////////////////////////////
    //  Utilities


    /** 
	@return first differing position from two equal-length integer
	arrays. -1 means that the two are exactly equal.  */
    private static int findDifference(int[]  a, int[] b){
	
	if (a.length!=b.length) {
	    //throw some ugly error ADDERROR/should never happen.
	    return -1;
	}
		
	int diff = a.length - 1;
       	
	while (a[diff] == b[diff]){
	    diff--;
	    if (diff<0) {
		break;
	    }
	}

	return diff;
    }
    



    /**Creates an integer array of the values of the SHA1Hash chunked
     * into the right size pieces.
     * TODO, make this work with variable bases and lengths.  
     * @return an integer array of the values of the SHA1Hash.  It's
     * useful for switching and comparing (as opposed to a straight
     * byte-array) */
    public int[] decodeGuid(SHA1Hash guid){




	int[]  returnArray = new int[LENGTH];
	byte[] guidBytes = guid.bytes();
	
	int raPosition = LENGTH;
	
	/*TODO -- to make this flexible, allow for X ints/byte, where
	  X = (ln 256)/(ln base);
	  
	  and for masking, it'd be ...
	  //assuming base 4
	  ra[p - 1] = ((gb[pos] <<< 6    )  >>> 6    )& BASE
	  ra[p - 2] = ((gb[pos] <<< 4    )  >>> 6    )& BASE
	  ra[p - 3] = ((gb[pos] <<< 2    )  >>> 6    )& BASE
	  ra[p - 4] = ((gb[pos] <<< 0    )  >>> 6    )& BASE
	  
	  left column  = 8 - index* ((ln X)/(ln 2))
	  right column = 8 - (ln X)/(ln 2)
	  

	  
	  recoded : 
	  
	  slotsPerByte = (ln 256)/(ln BASE );
	  int raPosition = LENGTH;
	  int pushLeft = ((ln X)/(ln 2)); 
	  int pushright =  8 - (ln X)/(ln 2);
	  
	  for (int gPosition = guidBytes.length - 1; gPosition >=0; gPosition--) {
	  for (int raOffset = 1; raOffset<=slotsPerByte; raOffset++){
	  returnArray[raPosition - raOffset] = 
	  ((guidBytes[gPosition] 
	  <<< pushLeft*raOffset)
	  >>>pushRight)
	  & 15;
	  }
	  
	  raPosition = raPosition - slotsPerByte;	    
	  }
	  
	  
	*/
	
	for (int gPosition = guidBytes.length - 1; gPosition >=0; gPosition--) {
	    
	    returnArray[raPosition - 1] = guidBytes[gPosition] & 15;
	    returnArray[raPosition - 2] = (guidBytes[gPosition] >>> 4) & 15; 
	    raPosition = raPosition - 2;
	}
	
	return returnArray;
    }
    
   



    ////////////////////////////////////////////////////////////
    // TEST code, relies on somewhat previous versions of the function interfaces. Commented out, not updated.


    



	
//      public static void main(String[] args) {
//  	try {
//  	    NodeId nodeid = new NodeId(5, InetAddress.getLocalHost());
//  	    NeighborTable nt = new NeighborTable(40,16,3,nodeid);
	    
//  	    // nt.testUtils();
//  	    nt.testAdd();
//  	    // nt.testHash();
	    
//  	} catch (UnknownHostException e) {}

	
//      }

    

//      public void testHash(){
//  	try {
//  	String a,b;
//  	SHA1Hash ah, bh;
//  	a = "hey";
//  	b = "he";
	
//  	ah = new SHA1Hash(a);
//  	System.out.println(b);
//  	b = "e"+"y";
//  	b = "h" +b;
//  	bh = new SHA1Hash(b);
//  	System.out.println(ah.fullString());
//  	System.out.println(bh.fullString());
	

//  	NodeId ha  = new NodeId(4, InetAddress.getLocalHost());
//  	NodeId hb  = new NodeId(4, InetAddress.getLocalHost());

//  	ah = new SHA1Hash(ha);
//  	bh = new SHA1Hash(hb);
//  	System.out.println(ah.fullString());
//  	System.out.println(bh.fullString());
//  	} catch (Exception e){}
//      }


//      public void testAdd(){

//  	Random r = new Random();
	
//  	int numNodes = 100;
//  	int numfind = 100;
//  	byte [] byteArray = new byte[LENGTH];
//  	SHA1Hash guid;
//  	Link nodeInfo;
//  	NodeId nodeid;

//  	SLADEBUG=false;
//  	BLDEBUG=false;
//  	GNNDEBUG=false;

//  	for (int i = 0; i < numNodes ; i++){
//  	    try {
//  		r.nextBytes(byteArray);
//  		guid = new SHA1Hash(byteArray);
//  		nodeid = new NodeId(i, InetAddress.getLocalHost());
//  		nodeInfo = new Link(guid,nodeid,r.nextInt());
//  		if (r.nextFloat()>.5){  //most of them are up.
//  		    nodeInfo.setUp();
//  		}
//  		else {
//  		    if (r.nextFloat()<.2){  //some get blacklisted before added.
//  			blacklist(nodeid);
			
//  		    } else {
//  			nodeInfo.setDown(); //the rest of them are marked down.
//  		    }
//  		}
//  		add(nodeInfo);

//  		nodeInfo.sent();     //updates clock for the nodes.
//  		nodeInfo.received(); //updates clock for the nodes.
//  		if ((r.nextFloat()<.1) && !nodeInfo.isBlacklisted()) {
//  		    //		    System.out.print("post-add");
//  		    blacklist(nodeid); //blacklist it after the fact.  Just to see what happens.
//  		}
//  	    } catch (UnknownHostException e) {
//  	    }
//  	}
//  	//long time = System.currentTimeMillis();
//  	markOld(4000);
//  	/*	time = System.currentTimeMillis() - time;
//  		System.out.println("Time to check 100,000 nodes for down : " +  time+ " milliseconds");
//  		float f = 100000/time;
//  		System.out.println("Checks/millisecond " +  f);
//  	*/
	
//  	//test getNextNode

//  	for (int i = 0; i < numfind ; i++){
//  	    try {
//  		r.nextBytes(byteArray);
//  		guid = new SHA1Hash(byteArray);
//  		nodeid = new NodeId(i, InetAddress.getLocalHost());
//  		//		nodeInfo = 
//  		    getNextNode(nodeid,39);
//  		    getNextNode(nodeid,38);
//  		    getNextNode(nodeid,37);
//  		    getNextNode(nodeid,36);
//  		    //if (nodeInfo!=null) {
//  		    //nodeInfo.sent();     //updates clock for the nodes.
//  		    //	}
//  	    } catch (UnknownHostException e) {}
//  	}    
	
//  	//Routing to self?  What happens?
//  	try {
//  	    nodeid = new NodeId(5,InetAddress.getLocalHost());
//  	    guid = new SHA1Hash(nodeid);
//  	    for (int i = 39; i >=0; i--){
//  		getNextNode(nodeid,i);
//  	    }
//  	} catch (UnknownHostException e) {}
	
	
//      }
    
    
    
    
    
//      public void testUtils(){
//  	//test code, 
//  	//read from file, insert nodes, do operations.
	
	
//  	///////////test utils
//  	//a and b are the same, c is different
	
//  	SHA1Hash a, b, c ;
//  	byte[] abyte =	{14,75,56,63,-15,51,1,-40,-28,64,-94,-55,23,14,5,127,-128,111,114,124};
//  	byte[] bbyte =	{14,75,56,63,-15,51,1,-40,-28,64,-94,-55,23,14,5,127,-128,111,114,124};
//  	byte[] cbyte =	{13,75,56,63,-15,11,5,10,-8,64,-94,-51,23,14,5,15,-18,11,114,124};

//  	a = new SHA1Hash(abyte);
//  	b = new SHA1Hash(bbyte);
//  	c = new SHA1Hash(cbyte);

//  	int[] adecode = decodeGuid(a);
//  	int[] bdecode = decodeGuid(b);
//  	int[] cdecode = decodeGuid(c);
	
//  	System.out.println("Testing decode -- here's what each one looks like");

//  	System.out.print("A = ");
//  	for (int i = 0; i <adecode.length;i++){
//  	    System.out.print(adecode[i]+" ");
//  	}
//  	System.out.println("");
	
//  	System.out.print("B = ");
//  	for (int i = 0; i <bdecode.length;i++){
//  	    System.out.print(bdecode[i]+" ");
//  	}
//  	System.out.println("");

//  	System.out.print("C = ");
//  	for (int i = 0; i <cdecode.length;i++){
//  	    System.out.print(cdecode[i]+" ");
//  	}
//  	System.out.println("");
	
	
//  	System.out.println("Testing findDifference");
//  	System.out.println("A and B differ on position " + findDifference(adecode,bdecode));
//  	System.out.println("A and C differ on position " + findDifference(adecode,cdecode));
	


//      }








}
















