/**
 * The parent class for all types of tapestry node-node communications.
 *
 * TODO<br>
 * get cloneable stuff working
 * testing/debugging
 * 
 * @author     Scott Honey
 * @version    $Id
 */



package ostore.tapestry;

import java.util.*;
import java.net.*;
import ostore.util.*;
import ostore.dispatch.*;
import ostore.network.*;
import mdw.sandStorm.api.*;
import mdw.sandStorm.lib.aSocket.*;


public class TapestryInternalMessage extends NetworkMessage {



    //    public NodeId peer;  from NetworkMessage   -- will change?
    //    public boolean inbound; from NetworkMessage --will change?



    protected SHA1Hash 	source;
    protected SHA1Hash 	destination;
    //    protected NodeId 	nextHop;  -- changed to "peer" to match inherited NetworkMessage
    protected int 	hop;

    
    protected int 	typeCode;
    
    protected boolean exactMatch;

    //lots of codes
/* This stuff should all be in util.Types
    public static final int ROUTE_DATA_MESSAGE  	    = 31000;
    public static final int ROUTE_SEARCH_MESSAGE 	    = 32000;
    public static final int ROUTE_SEARCH_REPLY_MESSAGE = 32000;
    
    public static final int ADVERTISE_MESSAGE  	= 41000;
    public static final int UNADVERTISE_MESSAGE  = 42000;

    public static final int NEIGHBOR_REQUEST_MESSAGE = 51000;
    public static final int NEIGHBOR_RESULT_MESSAGE  = 52000;

    public static final int ERROR_MESSAGE  = 61000;



*/

    
    public TapestryInternalMessage(int t_typeCode, SHA1Hash t_source, SHA1Hash t_destination, NodeId t_nextHop, int t_hop){
	
	typeCode = t_typeCode;
	source = t_source;
	destination = t_destination;
	peer = t_nextHop;  //from superclass
	hop = t_hop;
    }


	//TODO define type in typetable.
	public int type_code(){
		return ostore.util.Types.TAPESTRY_INTERNAL_MESSAGE;
	}
    
    
    public TapestryInternalMessage (){ 
	
    }    



    //////////////wire format stuff

    

    /** Order of bytes being packed == 
	type code
	source
	destination
	next hop
	hop #
	inbound
	exactMatch
	
	    remaining stuff will be packed by subclass.
     */
	
    public int to_bytes (byte [] data, int offset) {
	int old_offset = offset;
	offset += super.to_bytes (data, offset); //NetworkMessage performs useful tasks?
	offset +=ByteUtils.intToBytes (typeCode, data, offset);
	offset +=source.to_bytes(data, offset);
	offset +=destination.to_bytes(data, offset);
	offset +=peer.to_bytes(data,offset);
	offset +=ByteUtils.intToBytes (hop, data, offset);
	offset +=ByteUtils.boolToBytes (inbound, data, offset);
	offset +=ByteUtils.boolToBytes (exactMatch, data, offset);
	return offset - old_offset;
    }

    /** Order of bytes being unpacked == 
	type code
	source
	destination
	next hop
	hop #
	inbound
	exactMatch
    */

    public int from_bytes (byte [] data, int offset) {
	int old_offset = offset;
	offset += super.from_bytes (data, offset);//assuming NetworkMessage useful

	typeCode = ByteUtils.bytesToInt (data, offset);
	offset += 4;
	
	if (source==null) {
	    source = new SHA1Hash();
	}
	offset += source.from_bytes (data, offset);
	
	if (destination == null){
	    destination = new SHA1Hash ();
	}
	offset += destination.from_bytes (data, offset);
	
	if (peer == null) {
	    peer = new NodeId();
	}
	offset += peer.from_bytes(data,offset);

	hop = ByteUtils.bytesToInt (data, offset);
	offset += 4;
	
	inbound =  ByteUtils.bytesToBool (data, offset);
	offset += ByteUtils.SIZE_BOOL;

	exactMatch =  ByteUtils.bytesToBool (data, offset);
	offset += ByteUtils.SIZE_BOOL;

	return offset - old_offset;
    }





    /*possible factory for tapestry internal messages.
       read off type, then produce proper, then return it.
       I'm almost certain this type of functionality already exists in the system (perhaps the type table?)

    public TapestryInternalMessage makeMessage(byte[] data, int offset){
	TapestryInternalMessage message;
	int typeCode;

	typeCode = getTypeCode(data,offset);
	
	switch(typeCode) {
	case RouteDataMessage: {
	    message = new RouteDataMessage();
	    message.from_bytes(data,offset);
	    return message;
	    break; //not needed
	}
	case RouteSearchMessage: {

	    break;
	}
		    
	default: 
	    //error
	    break;
	}
	
	

    }
    */

    ////////////// accessors

    
    public int typeCode () {
	return typeCode;
    }


    /**assuming first on deck is the type code, as should be.*/

    public int getTypeCode(byte [] data, int offset){
	return ByteUtils.bytesToInt (data, offset);	
    }

    

    
    public String toString(){
	return ("TapestryInternalMessage: source "+source +
		" destination "+destination+
		" next hop "+peer+
		" hop# "+hop+
		(exactMatch ? " exact match" : "")+"\n");
    }
    
    
    public Object clone () throws CloneNotSupportedException {

    	CloneNotSupportedException e = new CloneNotSupportedException("not done yet");
	throw(e);
    }
    /*NetworkMessage result = (NetworkMessage) super.clone ();
      result.peer = peer;
      result.inbound = inbound;
      return result;*/

    // }
    


    
}



