/**
 * VerifiableObject
 *
 * @author     Ben Y. Zhao
 * @version    $Id: VerifiableObject.java,v 1.1 2001/06/01 01:08:47 s_honey Exp $
 */

package ostore.tapestry;

import ostore.util.*;
import java.io.*;

/**
 * VerifiableObject is an interface that any message that can be verified 
 * implements. The key method is verify() which defines an application
 * specific verification mechanism.
 * NOTE: the continuation-like mechanism for generating more tapestry
 * messages from verification is not yet done. It's necessary when the
 * required verification information is not available at the verifying 
 * Tapestry node.
 */
public interface VerifiableObject extends QuickSerializable {
    /**
     * Method that defines an application or message specific verification 
     * mechanism.  The Tapestry node simply passes the verifyFields class 
     * to this function, and it either issues a continuation-like call to 
     * generate more Tapestry messages requesting verification keys etc 
     * etc, or else it computes the verification, and returns the result.
     * @param vField    The QuickSerializable object which encapsulates 
     *                  the data necessary to compute verification
     * @return		The success or failure of the verification.
     * @throws VerificationException Verification needs more data.  How 
     *                               the continuation mechanism works and 
     *                               additional messages sent is not yet 
     *                               worked out is not yet figured out.
     */
    public boolean verify(QuickSerializable vFields) 
      throws VerificationException;
}

