/**
 *  TapestryLocationFailure.java
 *
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.tapestry.api;
import ostore.network.NetworkMessage;
import ostore.util.SecureHash;
import mdw.sandStorm.api.QueueElementIF;
import ostore.util.Types;
import ostore.util.TypeTable;
import ostore.util.QSException;
import ostore.util.ByteUtils;
import ostore.util.NodeId;

/**
 * An event which indicates that a <code>TapestryLocateMsg</code> was not
 * successful.  In other words, that the {@link TapestryLocateMsg} made its
 * way all the way root without getting a positive query result.
 *
 * <p>NOTE: This event should be treated in much the same way as a DNS
 * lookup failure.  It does not necessary indicate that the query
 * <i>shouldn't</i> have succeeded, only that it didn't.  For instance, an
 * intermediate node between the query source and the root could well cause
 * such a failure, perhaps maliciously.  <i>Use with caution; you have been
 * warned.</i>
 *
 * @author     Ben Y. Zhao
 * @version    $Id: TapestryLocateFailure.java,v 1.2 2002/01/28 20:12:57 ravenben Exp $
 */
public class TapestryLocateFailure extends NetworkMessage {

  /**
   * The hash of the original request. Used to distinquish this request.
   */
  public SecureHash req_hash;
  /**
   * The guid where the request was determined to have failed. Should be the
   * dynamically mapped root of the object ID originally requested for.
   */
  public SecureHash root;

  public SecureHash obj_guid;
  
  public TapestryLocateFailure (SecureHash root, SecureHash req_hash,
				SecureHash obj_guid, NodeId req) {
    super(req, false /*outbound*/);
    this.req_hash = req_hash;
    this.root = root;
    this.obj_guid = obj_guid;
  }
  
  public TapestryLocateFailure (byte [] buffer, int [] offset) 
    throws QSException {
    super(buffer, offset);
    int type_code = ByteUtils.bytesToInt(buffer, offset);
    root = (SecureHash) TypeTable.new_object(type_code, buffer, offset);
    type_code = ByteUtils.bytesToInt(buffer, offset);
    req_hash = (SecureHash) TypeTable.new_object(type_code, buffer, offset);
    type_code = ByteUtils.bytesToInt(buffer, offset);
    obj_guid = (SecureHash) TypeTable.new_object(type_code, buffer, offset);
  }
  
  public void to_bytes(byte [] buffer, int [] offset) {
    super.to_bytes(buffer, offset);
    ByteUtils.intToBytes(root.type_code(), buffer, offset);
    root.to_bytes(buffer, offset);
    ByteUtils.intToBytes(req_hash.type_code (), buffer, offset);
    req_hash.to_bytes(buffer, offset);
    ByteUtils.intToBytes(obj_guid.type_code(), buffer, offset);
    obj_guid.to_bytes(buffer, offset);
  }
  
  public int type_code () {
    return Types.TAPESTRY_LOCATE_FAILURE;
  }
}

