/**
 * TapestryLocationMsg.
 *
 * @author     Sean C. Rhea
 * @version    $Id: TapestryLocateMsg.java,v 1.4 2002/04/06 22:52:43 eaton Exp $
 */
package ostore.tapestry.api;
import ostore.util.SecureHash;
import ostore.util.Digestible;
import mdw.sandStorm.api.QueueElementIF;

/**
 * Messages sent from one OceanStore machine to one or more machines
 * avertising the given GUID and satisfying the given tag query.  The
 * intended functionality of this event (outgoing) is as follows.  The
 * message is routed toward the given {@link #guid} in the Tapestry mesh,
 * and at each hop the {@link #query} is applied to any backpointers
 * associated with the given GUID, along with the current {@link
 * #query_state}.  The query should return a result, and possibly modify
 * the query state as well.  The result can be one of {@link
 * TapestryQueryResultMatch} or {@link TapestryQueryResultNoMatch}.  A
 * matching result indicates which backpointers to follow, as well as
 * whether or not to continue up the tree.  A non-matching result causes
 * the query to continue towards the root without following any
 * backpointers.
 */
public abstract class TapestryLocateMsg 
implements QueueElementIF, Digestible {

    /**
     * If <code>inbound</code>, the sender; undefined otherwise.
     */
    public SecureHash peer;

     /**
     * Whether this message is being received (<code>true</code>) or sent
     * (<code>false</code>).
     */
    public boolean inbound;

    /**
     * The queried GUID.
     */
    public SecureHash guid;

    /**
     * The query to execute on each backpointer.  See the class comments,
     * above.
     */
    public TapestryQuery query;

    /**
     * The state to pass the query executed on each backpointer.  See the
     * class comments, above.
     */
    public TapestryQueryState query_state;

    /**
     * A unique ID for this message, in order that a
     * <code>TapestryLocateFailure</code> can be matched to the location
     * request; set to <code>null</code> if you do not want failure
     * notification.
     */
    public SecureHash id;

    /**
     * Construct with the given guid, query, and state; will not return a
     * <code>TapestryLocateFailure</code> if the location fails.  Use for
     * new outbound messages.
     */
    public TapestryLocateMsg (SecureHash guid, TapestryQuery query, 
	    TapestryQueryState query_state) {

	this.peer = null;
	this.inbound = false;
	this.guid = guid;
	this.query = query;
	this.query_state = query_state;
	this.id = null;
    }

    /**
     * Construct with the given guid, query, and state; will return a
     * <code>TapestryLocateFailure</code> with the same <code>id</code> if
     * the location fails.  Use for new outbound messages.
     */
    public TapestryLocateMsg (SecureHash guid, TapestryQuery query, 
	    TapestryQueryState query_state, SecureHash id) {

	this.peer = null;
	this.inbound = false;
	this.guid = guid;
	this.query = query;
	this.query_state = query_state;
	this.id = id;
    }

    /**
     * Empty constructor.  For use by derived types when reconstructing
     * themselves from a byte array.
     */
    public TapestryLocateMsg () {}
}

