/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.tapestry.impl;
import ostore.bloom.BloomQuery;
import ostore.util.ByteUtils;
import ostore.util.QuickSerializable;
import ostore.util.QSException;
import ostore.util.SecureHash;
import ostore.util.Types;
import ostore.util.TypeTable;

/**
 * The bloom filter part of a TapestryLocateMsg or TapestryRouteMsg.
 *
 * @author     Sean C. Rhea
 * @version    $Id: BloomWrapper.java,v 1.2 2002/04/06 22:52:43 eaton Exp $
 */
public class BloomWrapper extends BloomQuery {
    
    public QuickSerializable user_data;

    public BloomWrapper (SecureHash dest, QuickSerializable user_data) { 
	super (dest); 
	this.user_data = user_data;
    }

    public BloomWrapper (byte [] data, int [] offset) throws QSException {
        long type_code = ByteUtils.bytesToLong (data, offset);
	user_data = TypeTable.new_object (type_code, data, offset);
    }

    public void to_bytes (byte [] data, int [] offset) {
	ByteUtils.longToBytes (TypeTable.type_code(user_data), data, offset);
	user_data.to_bytes (data, offset);
    }

    public int type_code () { 
        throw new NoSuchMethodError();
	//return Types.TAPESTRY_BLOOM_WRAPPER; 
    }

    public String toString () {
	return "(BloomWrapper " + src + "->" + dest + " " +
	    (inbound ? (success ? "succeeded" : "failed")
	             : "outbound") + " " + user_data + ")";
    }
}

