/**
 * DynamicGatewayMsg.java
 *
 * @author Ben Y. Zhao
 * @version $Id: DynamicGatewayMsg.java,v 1.1 2002/01/10 07:09:48 ravenben Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.Types;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.ByteUtils;
import ostore.util.QSException;

public class DynamicGatewayMsg extends NetworkMessage {

  public SecureHash src;

  public DynamicGatewayMsg (NodeId peer, SecureHash src) {
    super (peer, false /*outbound*/);
    this.src = src;
  }

  public DynamicGatewayMsg (byte [] data, int [] offset) throws QSException {
    super (data, offset);
    int type = ByteUtils.bytesToInt(data, offset);
    src = (SecureHash) TypeTable.new_object(type, data, offset);
  }

  public void to_bytes(byte [] data, int [] offset) {
    super.to_bytes(data, offset);
    ByteUtils.intToBytes(src.type_code(), data, offset);
    src.to_bytes(data, offset);
  }

  public int type_code() {
    return Types.TAPESTRY_DYNAMIC_GATEWAY_MSG;
  }

  public Object clone () throws CloneNotSupportedException {
    DynamicGatewayMsg result = (DynamicGatewayMsg) super.clone();
    result.src = src;
    return result;
  }

  public String toString() {
    return "(DynamicGatewayMsg: src="+src+", peer="+peer+")";
  }

}
