/**
 * DynamicMCReqMsg.java
 *
 * @author Ben Y. Zhao
 * @version $Id: DynamicMCReqMsg.java,v 1.1 2002/04/05 06:18:36 ravenben Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.Types;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.ByteUtils;
import ostore.util.QSException;

public class DynamicMCReqMsg extends NetworkMessage {

  public SecureHash src;  //source of message
  public boolean request;     //is this request(0) or confirmation(1)

  public DynamicMCReqMsg (NodeId peer, SecureHash src, boolean request) {
    super (peer, false /*outbound*/);
    this.src = src;
    this.request = request;
  }

  public DynamicMCReqMsg (byte [] data, int [] offset) throws QSException {
    super (data, offset);
    int type = ByteUtils.bytesToInt(data, offset);
    src = (SecureHash) TypeTable.new_object(type, data, offset);
    request = ByteUtils.bytesToBool(data, offset);
  }

  public void to_bytes(byte [] data, int [] offset) {
    super.to_bytes(data, offset);
    ByteUtils.intToBytes(src.type_code(), data, offset);
    src.to_bytes(data, offset);
    ByteUtils.boolToBytes(request, data, offset);
  }

  public int type_code() {
    return 0;
  }

  public Object clone () throws CloneNotSupportedException {
    DynamicMCReqMsg result = (DynamicMCReqMsg) super.clone();
    result.src = src;
    result.request = request;
    return result;
  }

  public String toString() {
    if (request)
      return "(DynamicMCReqMsg: root-->"+src+", from/to-->"+peer+", REQUEST)";
    else 
      return "(DynamicMCReqMsg: root-->"+src+", from/to-->"+peer+", CONFIRM)";
  }

}
