/**
 * DynamicMapMsg.java
 *
 * @author Ben Y. Zhao
 * @version $Id: DynamicMapMsg.java,v 1.7 2001/12/31 00:43:34 ravenben Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.Types;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.ByteUtils;
import ostore.util.QSException;
import ostore.util.Carp;
import java.util.Iterator;
import java.util.ArrayList;

public class DynamicMapMsg extends NetworkMessage {

  public SecureHash src;
  public SecureHash dest;
  public ArrayList route_level; //single level (hop) of the routing table
  public int hop;   //A routes to B, hop(0) = A, ...

  public DynamicMapMsg (NodeId peer, SecureHash src, SecureHash dest,
			ArrayList route_level, int hop) {
    super (peer, false /*outbound*/);
    this.src = src;
    this.dest = dest;
    this.route_level = route_level;
    this.hop = hop;
  }

  public DynamicMapMsg (byte [] data, int [] offset) throws QSException {
    super (data, offset);
    int type = ByteUtils.bytesToInt(data, offset);
    src = (SecureHash) TypeTable.new_object(type, data, offset);
    type = ByteUtils.bytesToInt(data, offset);
    dest = (SecureHash) TypeTable.new_object(type, data, offset);
    
    SecureHash name;
    NodeId addr;
    type = ByteUtils.bytesToInt(data, offset);
    int size = ByteUtils.bytesToInt(data, offset);
    //    System.err.println("DynamicMapMsg with "+size+" GuidAddrs.");
    route_level = new ArrayList(size);
    //initial capacity >= 16 / 0.75
    for (int i =0 ; i < size; i++) {
      name = (SecureHash) TypeTable.new_object(type, data, offset);
      addr = new NodeId(data, offset);
      if (!name.isNull()) 
	//if null guid, means pair of guid/addr is useless
	route_level.add(new GuidAddr(name, addr));
      else 
	route_level.add(new GuidAddr(null, addr));
    }
    hop = ByteUtils.bytesToInt(data, offset);
  }

  /*  public QuickSerializable data_to_mac() {
    QuickSerializable [] x = {src, dest, user_data};
    return new QSArray(x);
    }*/

  public void to_bytes(byte [] data, int [] offset) {
    super.to_bytes(data, offset);
    ByteUtils.intToBytes(src.type_code(), data, offset);
    src.to_bytes(data, offset);
    ByteUtils.intToBytes(dest.type_code(), data, offset);
    dest.to_bytes(data, offset);

    ByteUtils.intToBytes(src.type_code(), data, offset);
    ByteUtils.intToBytes(route_level.size(), data, offset);

    SecureHash name;
    NodeId addr;
    GuidAddr ga;
    for (Iterator iter = route_level.iterator(); iter.hasNext(); ) {
      ga = (GuidAddr) iter.next();
      name = ga.getGuid();
      if (name == null) 
	src.nullHash().to_bytes(data, offset);
      else 
	name.to_bytes(data, offset);
      addr = ga.getAddr();
      if (addr == null)
	Carp.die("Should never happen!!!!");
      addr.to_bytes(data, offset);
    }

    ByteUtils.intToBytes(hop, data, offset);
    /*    ByteUtils.intToBytes(user_data.type_code(), data, offset);
	  user_data.to_bytes(data, offset);
	  if (mac == null)
	  ByteUtils.intToBytes(0, data, offset);
	  else {
	  ByteUtils.intToBytes(mac.type_code(), data, offset);
	  mac.to_bytes(data, offset);
	  }*/
  }

  public int type_code() {
    return Types.TAPESTRY_DYNAMIC_MAP_MSG;
  }

  public Object clone () throws CloneNotSupportedException {
    DynamicMapMsg result = (DynamicMapMsg) super.clone();
    result.src = src;
    result.dest = dest;
    result.hop = hop;
    //    result.user_data = user_data;
    result.route_level.addAll(route_level);
    //    result.mac = mac;
    return result;
  }

  public String toString() {
    String s = "(DynamicMapMsg:  hop= "+hop+" \n[";
    for (int i=0; i<route_level.size(); i++) {
      s += (GuidAddr) route_level.get(i) +", ";
    }
    return s;
  }

}
