/**
 * DynamicObjMapMsg.java
 *
 * @author Ben Y. Zhao
 * @version $Id: DynamicObjMapMsg.java,v 1.6 2002/03/27 00:08:07 ravenben Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.Types;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.ByteUtils;
import ostore.util.QSException;
import ostore.util.QuickSerializable;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.Set;
//import ostore.mac.MACedQS;

public class DynamicObjMapMsg extends NetworkMessage {

  public SecureHash src;
  public SecureHash dest; //don't think this is actually necessary
  public SecureHash objguid; //linkedsets of objects to node(s)
  public LinkedList pinfo;
  public int objcount;    //how many of these msgs are coming
  public int seq;         //which msg is this
  // if objcount == seq, means its the last message
  // if objcount == 0, means there are no objects to be sent over
  //  public MACedQS mac;

  public DynamicObjMapMsg (NodeId peer, SecureHash src, SecureHash dest,
			   SecureHash objguid, LinkedList pinfo,
			   int objcount, int seq) {
    super (peer, false /*outbound*/);
    this.src = src;
    this.dest = dest;
    this.objguid = objguid;
    this.pinfo = pinfo;   //linkedlist of publish info objects
    this.objcount = objcount;
    this.seq = seq;
  }

  public DynamicObjMapMsg (byte [] data, int [] offset) throws QSException {
    super (data, offset);
    int type = ByteUtils.bytesToInt(data, offset);
    src = (SecureHash) TypeTable.new_object(type, data, offset);
    type = ByteUtils.bytesToInt(data, offset);
    dest = (SecureHash) TypeTable.new_object(type, data, offset);
    type = ByteUtils.bytesToInt(data, offset);
    objguid = (SecureHash) TypeTable.new_object(type, data, offset);

    int numofpinfos = ByteUtils.bytesToInt(data, offset);
    PublishInfo pi;

    pinfo = new LinkedList();
    for (int i=0; i < numofpinfos; i++) {
      pi = new PublishInfo(data, offset);
      pinfo.add(pi);
    }

    objcount = ByteUtils.bytesToInt(data, offset);
    seq = ByteUtils.bytesToInt(data, offset);
  }

  public void to_bytes(byte [] data, int [] offset) {
    super.to_bytes(data, offset);
    ByteUtils.intToBytes(src.type_code(), data, offset);
    src.to_bytes(data, offset);
    ByteUtils.intToBytes(dest.type_code(), data, offset);
    dest.to_bytes(data, offset);
    ByteUtils.intToBytes(objguid.type_code(), data, offset);
    objguid.to_bytes(data, offset);

    ByteUtils.intToBytes(pinfo.size(), data, offset);

    PublishInfo pi;
    for (Iterator j=pinfo.iterator(); j.hasNext(); ) {
      pi = (PublishInfo) j.next();
      pi.to_bytes(data, offset);
    }

    ByteUtils.intToBytes(objcount, data, offset);
    ByteUtils.intToBytes(seq, data, offset);
  }

  public int type_code() {
    return Types.TAPESTRY_DYNAMIC_OBJMAP_MSG;
  }

  public Object clone () throws CloneNotSupportedException {
    DynamicObjMapMsg result = (DynamicObjMapMsg) super.clone();
    result.src = src;
    result.dest = dest;
    result.objguid = objguid;
    result.pinfo.addAll(pinfo);
    result.objcount = objcount;
    result.seq = seq;
    //    result.mac = mac;
    return result;
  }

  public String toString() {
    return "(DynamicObjMapMsg: #"+seq+" of "+objcount+"\n" + pinfo+ ")";
  }

}
