/*
 * DynamicTClient.java
 *
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.tapestry.impl;
import java.util.BitSet;
import java.util.Map;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.ArrayList;
import mdw.sandStorm.api.ConfigDataIF;
import mdw.sandStorm.api.EventHandlerException;
import mdw.sandStorm.api.EventHandlerIF;
import mdw.sandStorm.api.QueueElementIF;
import mdw.sandStorm.api.SinkIF;
import mdw.sandStorm.api.SinkException;
import mdw.sandStorm.api.StagesInitializedSignal;
import mdw.sandStorm.api.ProfilableIF;
import mdw.sandStorm.api.NoSuchStageException;
import mdw.sandStorm.internal.SinkProxy;
import ostore.util.Carp;
import ostore.util.Assert;
import ostore.util.Debug;
import ostore.util.DebugFlags;
import ostore.util.SecureHash;
import ostore.util.NodeId;
import ostore.util.TypeTable;
import ostore.util.QSException;
import ostore.dispatch.Classifier;
import ostore.dispatch.Filter;
import ostore.tapestry.api.TapestryInstance;

/**
 * A Tapestry client for dynamically configured networks.
 *
 * @author     Ben Y. Zhao
 * @version    $Id: DynamicTClient.java,v 1.31 2002/04/05 06:18:36 ravenben Exp $
 */
public class DynamicTClient implements EventHandlerIF, ProfilableIF {

    public void handleEvents(QueueElementIF items[]) 
	throws EventHandlerException {
	    for (int i=0; i < items.length; i++)
		handleEvent (items[i]);
	}

    public void destroy() throws Exception {
	// TODO
    }

    //  methods for ProfilableIF
    /* Specified by mdw.sandStorm.api.ProfilableIF */
    public int profileSize()
    {
	return _sink.size();
    }
    //  end methods for ProfilableIF

    public static final int McastTTL = 3;
    public static final float GATE_THRESH = (float) 0.15;
    private static final String class_tag = "DynamicTClient";
    private static final boolean DEBUG = DebugFlags.TAPESTRY_DYNAMIC_TCLIENT;

    private void dispatch (QueueElementIF item) {
	final String tag = class_tag + ".dispatch";
	if (DEBUG) Debug.printtagln (tag, "dispatching " +
		item.getClass () + " event: " + item + ".");
	try {
	    _classifier.dispatch(item);
	} catch (SinkException e) {
	    BUG (class_tag + ".dispatch: could not dispatch " + item);
	}
    }

    private static final void BUG (String msg) {
	Carp.die (msg + "  This is a bug in " +
		"ostore.tapestry.impl.DynamicTClient.  " +
		"Please let Ben know about it ASAP.");
    }

    public void init (ConfigDataIF config) throws Exception {

	final String tag = class_tag + ".init";
	Debug.turnOn (class_tag);

	_classifier = Classifier.getClassifier (
		new NodeId (config.getString ("node_id")));
	String stageName = config.getStage().getName();
	try 
	{
	  if(config.getBoolean("Profile"))
	  {
	    config.getManager().getProfiler().add("ostore.tapestry.impl.DynamicTClient", this);
	    SinkIF sink =
	      config.getManager().getStage(stageName).getSink();
	    _sink = new SinkProxy(sink,
				  config.getManager(),
				  config.getStage().getWrapper());
	  }
	  else
	    _sink =
	      config.getManager().getStage(stageName).getSink();
	} 
	catch (NoSuchStageException e) 
	{
	  Carp.die (tag + ": could not get own sink.");
	}


	_out_times = new HashMap ();
	_self_node_id = new NodeId (config.getString ("node_id"));
	_dynamic_route = config.getString("dynamic_route");
	_movingObjs = new HashMap();
	String gateway = config.getString("gateway");
	if (gateway != null)
	  _gate_node_id = new NodeId(gateway);

	Filter f = new Filter ();
	if (! f.requireType(Class.forName 
			    ("mdw.sandStorm.api.StagesInitializedSignal")))
	  BUG (tag + ": could not require type "+
	       "mdw.sandStorm.api.StagesInitializedSignal");
	_classifier.subscribe (f, _sink);

	if (_dynamic_route.equals("static")) {
	  f = new Filter();
	  if (!f.requireType(Class.forName 
			     ("ostore.tapestry.impl.RoutingTable$Available")))
	    BUG (tag + ": could not require type "+
		 "ostore.tapestry.impl.RoutingTable$Available");
	  _classifier.subscribe (f, _sink);
	}

	String [] inb_msg_types = {
	    //Dynamic Messages follow
	    "ostore.tapestry.impl.DynamicDMCastMsg",
	    "ostore.tapestry.impl.DynamicMCReqMsg",
	    "ostore.tapestry.impl.DynamicDMAckMsg",
	    "ostore.tapestry.impl.DynamicMapMsg",
	    "ostore.tapestry.impl.DynamicNotifyMsg",
	    "ostore.tapestry.impl.DynamicRequestMsg",
	    "ostore.tapestry.impl.DynamicObjMapMsg",
	    "ostore.tapestry.impl.DynamicObjReqMsg",
	    "ostore.tapestry.impl.DynamicSurrReqMsg",
	    "ostore.tapestry.impl.DynamicSurrMsg",
	    "ostore.tapestry.impl.DynamicPingMsg",
	    "ostore.tapestry.impl.DynamicPongMsg",
	    "ostore.tapestry.impl.DynamicGatewayMsg"
	};

	TypeTable.register_type("ostore.tapestry.impl.DynamicDMAckMsg");
	TypeTable.register_type("ostore.tapestry.impl.DynamicMCReqMsg");
	for (int i = 0; i < inb_msg_types.length; ++i) {
	    Filter filter = new Filter ();
	    if (! filter.requireType (Class.forName (inb_msg_types [i])))
		BUG (tag + ": could not require type " + inb_msg_types [i]);
	    if (! filter.requireValue ("inbound", new Boolean (true)))
		BUG (tag + ": could not require inbound = true for " 
		     + inb_msg_types [i]);
	    _classifier.subscribe (filter, _sink);
	    // if (DEBUG) Debug.printtagln(tag, "subscribing to inbound "+
	    //			inb_msg_types[i]);
	}

	if (_dynamic_route == null) {
	  BUG (tag+ ": missing dynamic_route config argument");
	} else if (_dynamic_route.equals("static")) {
	  Filter filter = new Filter ();
	  if (! filter.requireType(
		Class.forName("ostore.tapestry.impl.Router$ObjectMap")))
	    BUG (tag + ": could not require type " + 
		 "ostore.tapestry.impl.Router$ObjectMap");
	  Classifier.getClassifier(_self_node_id).subscribe (filter, _sink);
	}	  
	
    }

    protected void handle_stages_init_signal () {
	final String tag = class_tag + ".handle_stages_init_signal";

	if (DEBUG) Debug.printtagln (tag,
		"processing StagesInitializedSignal" );

	_self_guid = TapestryInstance.instance (_self_node_id).self_guid ();
	_self_digits = RoutingTable.digits (_self_guid);

	timer = 0;
	if (_dynamic_route.equals("dynamic")) {
	  _routing_tbl = new RoutingTable(_self_node_id);
	  _route_bitmap = new BitSet(_routing_tbl.digit_values ());
	  _backpointers = new HashMap();
	  _gateway_ready = false;
	  _gateway_pings = new HashMap(_routing_tbl.digit_values());
	  DynamicPingMsg outPing = new DynamicPingMsg(_gate_node_id);
	  _gateway_pings.put(_gate_node_id, 
			     new Long(System.currentTimeMillis()));
	  dispatch(outPing);
	  timer = System.currentTimeMillis();
	  DynamicGatewayMsg gateway_msg = 
	    new DynamicGatewayMsg(_gate_node_id, _self_guid);
	  //	  DynamicRequestMsg gateway_msg = 
	  //	    new DynamicRequestMsg(_gate_node_id, _self_guid, 0);
	  dispatch(gateway_msg);
	  //	  StaticPingMsg spm = new StaticPingMsg(_gate_node_id);
	  //	  dispatch(spm);
	} else 
	  _gateway_ready = true;
    }

    private void handle_routing_table_available (RoutingTable.Available msg) {

	final String tag = class_tag + ".handle_routing_table_available";
	if (DEBUG) {
	  Debug.printtagln (tag, "processing RoutingTable.Available" );
	  if (_dynamic_route.equals("dynamic")) {
	    Debug.printtagln(tag, "unexpected RoutingTable.Available");
	  }
	}
	Assert.assert (_routing_tbl == null, tag + ": got two versions " +
		"of the routing table.");

	_routing_tbl = msg.table;
	if (_dynamic_route.equals("static")) {
	  System.out.println("Tapestry: ready.\n");
	  System.out.flush();
	}
    }

  private void handle_object_map(Router.ObjectMap msg) {
    final String tag = class_tag+".handle_object_map";
    if (DEBUG) Debug.printtagln(tag, "processing ObjectMap "+msg);
    if (DEBUG) Assert.assert(_backpointers == null, 
			     tag+": got two versions of object map.");
    _backpointers = msg.objmap;
  }

  //informing this current node about new node with nodeID
  //notifier and guid notGuid
  private void handle_dynamic_notify(NodeId notifier, SecureHash notGuid) {

    //TODO: if the node is added, either to fill a hole or as an optimization,
    //we need to notify the node to add us to its reverse link list
      final String tag = class_tag + ".handle_dynamic_notify";
      if (_self_node_id.equals(notifier)) {
	if (DEBUG) 
	  Debug.printtagln(tag, "Received notify for self, ignoring.");
	return;
      }
      
      DynamicPingMsg ping = new DynamicPingMsg(notifier);
      if (_out_times.containsKey(notifier)) {
	//this is useful for cases where we get multiple notifymsgs
	//via different routes for the same guy
	if (DEBUG) Debug.printtagln(tag, "Receiving notify msg for a"+
				    " new node already pinged. Dropped.");
	return;
      }
      _out_times.put(notifier, new Long(System.currentTimeMillis()));
      if (_nodeid_to_guid == null) _nodeid_to_guid = new HashMap();
      if (!_nodeid_to_guid.containsKey(notifier)) {
	_nodeid_to_guid.put(notifier, notGuid);
      } else if (DEBUG)
	Debug.printtagln(tag, "Nodeid_to_guid entry already exists.");
      dispatch(ping);
      //received notify, ping back to measure latency
  }

    public void handleEvent(QueueElementIF item) 
    throws EventHandlerException {
        
	String tag = class_tag + ".handleEvent";
	if (DEBUG) Debug.printtagln (tag, "got " + item.getClass () + 
		" event: " + item + ".");

	if (item instanceof StagesInitializedSignal) {
	    handle_stages_init_signal ();
	} 
	else if (item instanceof RoutingTable.Available) {
	    handle_routing_table_available ((RoutingTable.Available) item);
	} else if (item instanceof Router.ObjectMap) {
	  handle_object_map((Router.ObjectMap) item);
	} else if (item instanceof DynamicGatewayMsg) {
	  tag = class_tag + ".handling DynamicGatewayMsg";
	  //Handling gateway request, send back list of neighbor(0) nodes
	  //in dynamicMapMsg
	  DynamicGatewayMsg msg = (DynamicGatewayMsg) item;
	  
	  if (DEBUG) {
	    Debug.printtagln(tag, "Handling DynamicGatewayMsg: "+msg);
	  }
	  ArrayList al = new ArrayList (_routing_tbl.digit_values());
	  NodeId entry;
	  for (int i=0; i<_routing_tbl.digit_values (); i++) {
	    entry = _routing_tbl.read (0, i);
	    if (entry != null && !entry.equals(_self_node_id)) 
	      al.add(new GuidAddr(_routing_tbl.node_id_to_guid (entry), 
				  entry));
	  }
	  DynamicMapMsg route_level = 
	    new DynamicMapMsg(msg.peer, _self_guid,
			      msg.src, al, 0);
	  dispatch(route_level);

	} else if (item instanceof DynamicRequestMsg) {
	  tag = class_tag + ".handling DynamicRequestMsg";
	  DynamicRequestMsg msg = (DynamicRequestMsg) item;
	  if (DEBUG) {
	    Debug.printtagln(tag, "Handling DynamicRequestMsg: "+msg);
	    Debug.printtagln(tag, "Routingtable = "+_routing_tbl);
	  }
	  ArrayList al = new ArrayList(_routing_tbl.digit_values ());
	  NodeId entry;
	  for (int i=0; i<_routing_tbl.digit_values (); i++) {
	    entry = _routing_tbl.read (msg.hop, i);
	    if (DEBUG) Debug.printtagln(tag, "hop="+msg.hop+
					", add GA entry="+entry);
	    if (entry == null) al.add(new GuidAddr(null, _self_node_id));
	    else al.add(new GuidAddr(_routing_tbl.node_id_to_guid (entry), 
				     entry));
	  }
	  DynamicMapMsg route_level = 
	    new DynamicMapMsg(msg.peer, _self_guid,
			      msg.src, al, msg.hop);
	  dispatch(route_level);

	} else if (item instanceof DynamicMapMsg) {
	  tag = class_tag + ".handling DynamicMapMsg";
	  DynamicMapMsg msg = (DynamicMapMsg) item;
	  if (DEBUG) Debug.printtagln(tag, "Handling DynamicMapMsg: "+ msg);
	  
	  if (!_gateway_ready) {
	    //for each entry in the map, do a ping
	    //if any result is significantly better than the gateway, repeat
	    int j = msg.route_level.size();
	    NodeId addr;
	    GuidAddr ga;
	    DynamicPingMsg ping;
	    for (int i =0; i < j; i++) {
	      ga = (GuidAddr) msg.route_level.get(i);
	      addr = ga.getAddr();
	      //we expect nothing but full entries to be in this msg
	      if (DEBUG) Debug.printtagln(tag, "Pinging possible gateway: "+
					  addr);
	      _gateway_pings.put(addr, new Long(System.currentTimeMillis()));
	      ping = new DynamicPingMsg(addr);
	      dispatch(ping);
	    }
	    return;
	  } else if (_route_bitmap.get(msg.hop)) {
	    if (DEBUG)
	      Debug.printtagln(tag, "Got DynamicMapMsg :"+msg+
			       ", we have this level already, dropping.");
	  } else {
	    _route_bitmap.set(msg.hop);
	    GuidAddr ga;
	    SecureHash name;
	    NodeId addr;
	    int j = msg.route_level.size();
	    for (int i=0; i<j; i++) {
	      ga = (GuidAddr) msg.route_level.get(i);
	      if (ga.getGuid() != null)
		_routing_tbl.write (msg.hop, i, ga.getAddr(), ga.getGuid(),
				    new Long(Long.MAX_VALUE));
	      //TODO, make this better!!
	    }
	  }
	  if (DEBUG) Debug.printtagln(tag, _routing_tbl.toString());
	  //now done copying the routing level into _routing_table
	  //TODO: optimize this level
	  
	  //now request next level
	  NodeId next = _routing_tbl.read (msg.hop, _self_digits[msg.hop]);
	  if (DEBUG) Debug.printtagln(tag, "Reading table: hop="+msg.hop+", "+
				      "value="+_self_digits[msg.hop]+", "+
				      "result="+next);
							   
	  if (next == null) {
	    //do surrogate routing to figure out
	    //who the actual surrogate is..., then
	    //move onto the next stage of insertion
	    next = _routing_tbl.surrogate(
		    _self_digits, _self_digits[msg.hop], msg.hop);
	    if (next == null) {
	      //found surrogate, now we can addself to own routing map
	      //then move on to next stage of insertion
	      _routing_tbl.addSelf(_self_guid);
	      _surrogate = msg.peer;
	      DynamicObjReqMsg dorm =
		new DynamicObjReqMsg(msg.peer, _self_guid, _self_guid,
				   DynamicObjReqMsg.REQUEST);
	      dispatch(dorm);
	    } else { //need to send more messages to find surrogate
	      DynamicSurrReqMsg dsrm =
		new DynamicSurrReqMsg(next, _self_guid, msg.hop+1);
	      dispatch(dsrm);
	    }
	    return;
	  } else {
	    DynamicRequestMsg next_msg =
	      new DynamicRequestMsg(next, _self_guid, msg.hop+1);
	    dispatch(next_msg);
	    //send request for next stage of the routing table
	  }

	} else if (item instanceof DynamicSurrReqMsg) {
	  tag = class_tag + ".handling DynamicSurrReqMsg";
	  DynamicSurrReqMsg msg = (DynamicSurrReqMsg) item;
	  if (DEBUG) Debug.printtagln(tag, "Handling DynamicSurrReqMsg: "+msg);

	  int [] dest_digits = RoutingTable.digits(msg.src);
	  NodeId next = _routing_tbl.surrogate(
		  dest_digits, dest_digits[msg.hop], msg.hop);
	  DynamicSurrMsg dsm;
	  if (next == null) {
	    dsm = new DynamicSurrMsg(msg.peer, _self_guid, _self_node_id,
				     msg.hop);
	  } else if (next.equals(_self_node_id)) {
	    int newhop = msg.hop+1;
	    while (next.equals(_self_node_id)) {
	      next = _routing_tbl.surrogate(
		      dest_digits, dest_digits[newhop], newhop);
	      newhop++;
	    }
	    if (next == null)
	      dsm = new DynamicSurrMsg(msg.peer, _self_guid, _self_node_id,
				       newhop-1);
	    else 
	      dsm = new DynamicSurrMsg(msg.peer, _self_guid, next, newhop-1);
	  } else {
	    dsm = new DynamicSurrMsg(msg.peer, _self_guid, next, msg.hop);
	  }
	  dispatch(dsm);

	} else if (item instanceof DynamicSurrMsg) {
	  tag = class_tag + ".handling DynamicSurrMsg";
	  DynamicSurrMsg msg = (DynamicSurrMsg) item;
	  if (DEBUG) Debug.printtagln(tag, "Handling DynamicSurrMsg: "+ msg);

	  if (DEBUG) 
	    Debug.printtagln(tag, "SurrResp: nextAddr="+msg.nextAddr+
			     ", peer="+msg.peer);
	  if (msg.nextAddr.equals(msg.peer)) { //found surrogate
	    //found surrogate, now we can addself to own routing map
	    //then move on to next stage of insertion
	    _routing_tbl.addSelf(_self_guid);
	    _surrogate = msg.peer;
	    DynamicObjReqMsg dorm =
	      new DynamicObjReqMsg(msg.peer, _self_guid, _self_guid,
				   DynamicObjReqMsg.REQUEST);
	    dispatch(dorm);
	    //TODO: optimize new routing table

	  } else { //keep surrogate routing
	    DynamicSurrReqMsg dsrm =
	      new DynamicSurrReqMsg(msg.peer, _self_guid, msg.hop+1);
	    dispatch(dsrm);
	  }

	} else if (item instanceof DynamicObjReqMsg) {
	  tag = class_tag + ".handling DynamicObjReqMsg";
	  DynamicObjReqMsg msg = (DynamicObjReqMsg) item;
	  if (DEBUG) Debug.printtagln(tag, "Handling DynamicObjReqMsg: "+ msg);

	  if (msg.msgtype == DynamicObjReqMsg.CONFIRM) {
	    //msg to confirm that sent objects were received
	    if (_movingObjs.containsKey(msg.dest)) {
	      int count = ((Integer) _movingObjs.get(msg.dest)).intValue();
	      _movingObjs.remove(msg.dest);
	      if (count > 0)  //counting # of objs txnfered and confirmed
		_movingObjs.put(msg.dest, new Integer(count - 1));
	    } else {
	      if (DEBUG) Debug.printtagln(tag, "Moved object confirmation on"+
					  " object we don't have!!");
	    }
	  } else {
	    //go through list of objects and examine which need to
	    //be moved to new surrogate, then put them into
	    //movingObjs with the new node's guid as index
	    int [] olddigits = RoutingTable.digits(_self_guid);
	    int [] newdigits = RoutingTable.digits(msg.dest);
	    SecureHash objguid;
	    int count=1;
	    DynamicObjMapMsg domm;
	    LinkedList l = new LinkedList();
	    if (DEBUG) {
	      if (_backpointers == null)
		Debug.printtagln(tag, "_backpointers is null");
	      if (_backpointers.keySet() == null) 
		Debug.printtagln(tag, "_backpointers.keySet() is null");
	    }

	    for(Iterator i = _backpointers.keySet().iterator();
		i.hasNext();) {
	      objguid = (SecureHash) i.next();
	      if (moveObj(olddigits, newdigits, objguid)) {
		l.add(objguid);
	      }
	    }
	    for (Iterator i = l.iterator(); i.hasNext(); count++) {
	      objguid = (SecureHash) i.next();
	      domm = new 
		DynamicObjMapMsg(msg.peer, _self_guid, msg.dest, objguid,
				 (LinkedList) _backpointers.get(objguid),
				 l.size(), count);
	      dispatch(domm);
	      if (DEBUG)
		Debug.printtagln(tag, "moving object "+objguid+
				 " to new node.");
	    }
	    if (l.size() == 0) {
	      domm = new DynamicObjMapMsg(msg.peer, _self_guid, msg.dest,
					  _self_guid.nullHash(),
					  new LinkedList(),
					  0, 0);
	      dispatch(domm);
	      if (DEBUG)
		Debug.printtagln(tag, "sending null message for objrequest.");
	    }
	    if (l.size() > 0)
	      _movingObjs.put(msg.dest, new Integer(l.size()));
	  }

	} else if (item instanceof DynamicObjMapMsg) {
	  tag = class_tag + ".handling DynamicObjMapMsg";
	  DynamicObjMapMsg msg = (DynamicObjMapMsg) item;
	  if (DEBUG) Debug.printtagln(tag, "Handling DynamicObjMapMsg: "+ msg);

	  if (msg.objcount > 0) {  //in other words, not just a null msg
	    LinkedList sss;
	    if (_backpointers.containsKey(msg.objguid)) {
	      sss = (LinkedList) _backpointers.get(msg.objguid);
	      sss.addAll(msg.pinfo);
	    } else _backpointers.put(msg.objguid, msg.pinfo);
	    
	    //got object from msg, now reply and confirm
	    DynamicObjReqMsg dorm = 
	      new DynamicObjReqMsg(msg.peer, _self_guid, _self_guid,
				   DynamicObjReqMsg.CONFIRM);
	    dispatch(dorm);
	  }

	  if (msg.objcount == msg.seq) { //last of objects transferred
	    //send the object backpointers to router
	    dispatch(new Router.ObjectMap(_backpointers));
	    
	    timer = System.currentTimeMillis() - timer;
	    System.err.println("Time to integration in ms: "+timer);
	    //now that we've got all objects, move on to notification
	    
	    //notify "hole" nodes to fill in entry for this
	    DynamicMCReqMsg req = 
	      new DynamicMCReqMsg(msg.peer, _self_guid, true); //request
	    dispatch(req);
	  }
	} else if (item instanceof DynamicMCReqMsg) {
	  DynamicMCReqMsg msg = (DynamicMCReqMsg) item;
	  tag = class_tag + ".handling DynamicMCReqMsg";
	  if (!msg.request) {   //msg is a confirmation, not a request
	    //routing table should be more or less done
	    //now lets send the reference to Router
	    if (DEBUG)
	      Debug.printtagln(tag, 
			       "got confirmation, proceeding w/ insertion.");
	    dispatch(new RoutingTable.Available(_routing_tbl));
	    
	    TapestryReadyMsg ready = new TapestryReadyMsg();
	    dispatch(ready);

	    //need to do a limited multicast notify for nearby nodes to
	    //optimize their routing tables
	  } else {
	    // we're the surrogate, we need to start the direct MC
	    int hop;
	    int [] selfguidint = RoutingTable.digits(_self_guid);
	    int [] destguidint = RoutingTable.digits(msg.src);
	    if (DEBUG)
	      Debug.printtagln(tag,
			       "selfInts:"+_self_guid+
			       " destInts:"+msg.src);

	    //To tell this node to check its distance to msg.mcastname
	    // to see if it's good for table insertion/replacement
	    //we do this by calling handle_dynamic_notify
	    handle_dynamic_notify(msg.peer, msg.src);

	    for (hop=0; selfguidint[hop] == destguidint[hop]; hop++) {
	    }
	    //if we get an ArrayoutofBounds exception, something's really wrong

	    //start directed multicast with no TTL
	    if (DEBUG)
	      Debug.printtagln(tag,
			       "Starting multicast for guid="+_self_guid);
	    int i = multicast(hop, -1, 
		      new DynamicDMCastMsg(null, _self_guid, _self_node_id,
					   msg.src, msg.peer, -1, -1));
	    if (i==0) { //we're the only surrogate, multicast unnecessary
	      if (DEBUG)
		Debug.printtagln(tag,
				 "we're only surrogate, mcast unnecessary");
	      DynamicMCReqMsg conf =
		new DynamicMCReqMsg(msg.peer, _self_guid, false);
	      dispatch(conf);
	    }
	  }
	} else if (item instanceof DynamicNotifyMsg) {
	  tag = class_tag + ".handling DynamicNotifyMsg";
	  DynamicNotifyMsg msg = (DynamicNotifyMsg) item;
	  if (DEBUG) Debug.printtagln(tag, "Handling DynamicNotifyMsg: "+ msg);
	  handle_dynamic_notify(msg.peer, msg.src);
	} else if (item instanceof DynamicDMCastMsg) {
	  tag = class_tag + ".handling DynamicDMCastMsg";
	  DynamicDMCastMsg msg = (DynamicDMCastMsg) item;
	  if (DEBUG) Debug.printtagln(tag, "Handling DynamicDMCastMsg: "+ msg);
	  int hop = msg.step+1;
	  int ttl = msg.ttl-1;
	  NodeId next;
	  int count;

	  count = multicast(hop, ttl, msg);

	  //To tell this node to check its distance to msg.mcastname
	  // to see if it's good for table insertion/replacement
	  //we do this by calling handle_dynamic_notify
	  handle_dynamic_notify(msg.newAddr, msg.newGuid);

	  //if we're a leaf of the mcast tree, then ack now
	  if (count == 0) {
	    DynamicDMAckMsg ack = new 
	      DynamicDMAckMsg(msg.peer, msg.src, 
			      msg.newGuid, msg.newAddr, msg.step, msg.ttl);
	    if (DEBUG)
	      Debug.printtagln(tag, "Leaf node here, sending MCastAck to "+
			       msg.peer);
	    dispatch(ack);
	  } else _mcast_parent = msg;
	} else if (item instanceof DynamicDMAckMsg) {
	  tag = class_tag + ".handling DynamicDMAckMsg";
	  DynamicDMAckMsg msg = (DynamicDMAckMsg) item;
	  if (DEBUG) Debug.printtagln(tag, "Handling DynamicDMAckMsg: "+msg);
	  
	  if (_mcast_waiting == null || !_mcast_waiting.contains(msg.peer)) {
	    if (DEBUG) {
	      Debug.printtagln(tag, "Dropping unexpected DMAckMsg. "+
			       msg);
	      System.err.println("MCast_waiting: "+_mcast_waiting+
				 " msg.src="+msg.src);
	    }
	  } else {
	    _mcast_waiting.remove(msg.peer);
	    if (_mcast_waiting.size() == 0) {
	      if (_mcast_parent == null) {
		DynamicMCReqMsg confirm = 
		  new DynamicMCReqMsg(msg.newAddr, _self_guid, false);
		dispatch(confirm);
	      } else {
		DynamicDMAckMsg ack = new
		  DynamicDMAckMsg(_mcast_parent.peer, _mcast_parent.src,
				  _mcast_parent.newGuid, _mcast_parent.newAddr,
				  _mcast_parent.step, _mcast_parent.ttl);
		dispatch(ack);
	      }
	    }
	  }
	} else if (item instanceof DynamicPingMsg) {
	  tag = class_tag + ".handling DynamicPingMsg";
	  DynamicPingMsg msg = (DynamicPingMsg) item;
	  if (DEBUG) Debug.printtagln(tag, "Handling DynamicPingMsg: "+msg);
	  dispatch (new DynamicPongMsg (msg.peer));

	} else if (item instanceof DynamicPongMsg) {
	  tag = class_tag + ".handling DynamicPongMsg";
	  long recv_time = System.currentTimeMillis();
	  if (DEBUG) Debug.printtagln(tag, "processing DynamicPongMsg");

	  DynamicPongMsg msg = (DynamicPongMsg) item;

	  long send_time;
	  if (_gateway_ready) {
	    if (_out_times == null || !_out_times.containsKey(msg.peer)) {
	      if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " +			     "Got dynamicPongMsg from unexpected source. "+
					  "Dropping it");
	      return;
	    }
	    send_time = ((Long) _out_times.remove(msg.peer)).longValue();
	  } else {  //_gateway not ready
	    if (_gateway_pings == null || 
		!_gateway_pings.containsKey(msg.peer)) {
	      if (DebugFlags.LOG_BAD_NETWORK_EVENTS) 
		Carp.out (tag + ": " +"Got unexpected DynamicPong: "+msg+
				 ", dropping it.");
	      return;
	    } //end if
	    send_time = ((Long) _gateway_pings.remove(msg.peer)).longValue();
	  } // end else
	  System.out.println("1");

	  long delta = recv_time - send_time;
	  if (DEBUG) {
	    if (_gateway_ready) 
	      Debug.printtagln(tag, "_out_times.size() = "+ _out_times.size());
	    else 
	      Debug.printtagln(tag, "_gateway_pings.size() = "+
			       _gateway_pings.size());
	  }

	  if (_gateway_ready) {
 	    System.out.println("2");
	    SecureHash newguid = (SecureHash) _nodeid_to_guid.get(msg.peer);
	    _routing_tbl.optimize(delta, msg.peer, newguid);
	  } else {
	    //TODO: put delta into map of measured gateway cand. distances
	    //compare to entry for _gate_node, if much better, then
	    //send dynamicRequest to new entry
	    if (_gateway_times == null)
	      _gateway_times = new HashMap();
	    _gateway_times.put(msg.peer, new Long(delta));
	    if (_gateway_pings.size()==0 && !msg.peer.equals(_gate_node_id)) {
	      if (DEBUG)
		Debug.printtagln(tag, "Got all gateway pings, now searching"+
				 " for new improved gateway.");
	      //now want to look through the times and find the best
	      Long best = new Long(Long.MAX_VALUE);
	      Long current;
	      NodeId next;
	      NodeId bestNid = _gate_node_id;
	      for (Iterator i=_gateway_times.keySet().iterator();
		   i.hasNext();) {
		next = (NodeId) i.next();
		current = (Long) _gateway_times.get(next);
		if (best.compareTo(current) < 0) {
		  bestNid = next;
		  best = current;
		}
	      } //end for Iterator
	      if (!bestNid.equals(_gate_node_id) &&
		  best.longValue() < 
		  (((Long) _gateway_times.get(_gate_node_id)).longValue() *
		   (1-GATE_THRESH))) {
		if (DEBUG) Debug.printtagln(tag, "Found new gateway: "+
					    bestNid);
		_gate_node_id = bestNid;
		_gateway_pings = new HashMap(_routing_tbl.digit_values());
		_gateway_times = new HashMap();
		_gateway_times.put(_gate_node_id, best);
		DynamicGatewayMsg g = 
		  new DynamicGatewayMsg(bestNid, _self_guid);
		dispatch(g);
	      } else {
		if (DEBUG) Debug.printtagln(tag, "No better gateway found.");
		_gateway_ready = true;
		DynamicRequestMsg req = 
		  new DynamicRequestMsg(_gate_node_id, _self_guid, 0);
		dispatch(req);
	      }
	    } //end if size == 0

	  } //end else (_gateway not ready)

	} else if (DEBUG)
	  Carp.die("ERROR in DynamicTClient: unrecognized incoming message.");
    }

  /**
   * Function that sends out a directed multicast to the relevant 
   * set of nodes which should know about its insertion into the network.
   * The node goes through all higher-level nodes in the routing table
   * starting with level equal to argument (hop) and sends a DMCastMsg to
   * them.  All of them are added to the _mcast_waiting list. Finally, a
   * counter is returned that specified # of DMCastMsgs sent out.
   */
  private int multicast(int hop, int ttl, DynamicDMCastMsg msg) {
    NodeId next;
    DynamicDMCastMsg nextmsg;
    int count = 0;
    String tag = class_tag + ".multicast";

    if (_mcast_waiting == null)
      _mcast_waiting = new HashSet();
    if (DEBUG) Debug.printtagln(tag,
				"entered multicast with hop="+hop+" ttl="+ttl);
    for (int j=0; j<_routing_tbl.DIGITS_PER_GUID-hop; j++) {
      for (int i = 0; i<_routing_tbl.digit_values(); i++) {
	next = _routing_tbl.read(hop+j, i);
	if (next != null && !next.equals(_self_node_id)) {
	  nextmsg = new DynamicDMCastMsg(next, msg.src, msg.srcAddr,
					 msg.newGuid, msg.newAddr,
					 hop, ttl);
	  _mcast_waiting.add(next);
	  if (DEBUG)
	    System.out.println("DynamicTClient:multicast"+
			       "Adding to mcast_waiting "+ next);
	  dispatch(nextmsg);
	  count++;
	}
      }
    }

    return count;
  }

  public boolean moveObj(int [] olddigits, int [] newdigits,
			 SecureHash objGuid) {
    //do we move the object to the new guy
    int i,j;
    int [] objdigits = RoutingTable.digits(objGuid);

    for(i=0; i<RoutingTable.DIGITS_PER_GUID; i++)
      if (olddigits[i] != objdigits[i])
	break;
    for(j=0; j<RoutingTable.DIGITS_PER_GUID; j++)
      if (newdigits[j] != objdigits[j])
	break;
    if (j > i) return true;
    else if (j < i) return false;

    // i==j
    while (olddigits[i] == newdigits[i]) i++;

    j = (objdigits[i] == RoutingTable.DIGIT_VALUES-1) ? 0 : objdigits[i];
    while (true) {
      if (j == olddigits[i]) return false;
      if (j == newdigits[i]) return true;
      j = (j == RoutingTable.DIGIT_VALUES - 1) ? 0 : j+1;
    }      

    /*    for (j=0; j<_routing_tbl.digits_per_guid (); j++) {
      if (objdigits[j] == olddigits[i]) return false;
      if (objdigits[j] == newdigits[i]) return true;
      } */

  }

  private long timer;
  private boolean _gateway_ready;
  private HashMap _gateway_pings;
  private HashMap _gateway_times;
  private HashSet _mcast_waiting;
  private DynamicDMCastMsg _mcast_parent;
  private String _dynamic_route;
  private SinkIF _sink;
  private Map _backpointers;  //mapping of objGuids->sets of publishInfo
  private HashMap _movingObjs;    //temp storage for objects in transit
  private BitSet _route_bitmap; //whether a level is filled in or not
  private NodeId _gate_node_id;
  private NodeId _surrogate;
  private SecureHash _self_guid;
  private int [] _self_digits;
  private Map _out_times;
  private Map _nodeid_to_guid;  //mapping nodeids to guids
  private RoutingTable _routing_tbl;
  private NodeId _self_node_id;
  private Classifier _classifier;
}

