/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.tapestry.impl;
import soss.core.BuilderIF;
import ostore.util.Carp;
import mdw.sandStorm.main.SandstormConfig;

/**
 * An interface for building config files.
 *
 * <p>Must have a constructor of the form 
 * <code>public Builder (String [] args)</code>.
 *
 * @author     Sean C. Rhea
 * @version    $Id: FederationBuilder.java,v 1.1 2001/12/17 06:43:37 srhea Exp $
 */
public class FederationBuilder implements BuilderIF {

    public FederationBuilder (String [] args) {}

    public SandstormConfig build_node (String [] args) throws Exception {

	if (args.length != 2) {
	    Carp.die ("usage: FederationBuilder <node id> <num nodes>");
	}

	String [] init_args = {"node_id=" + args [0], "nodes=" + args [1]};
	SandstormConfig result = new SandstormConfig ();
	result.addStage ("Federation", 
		"ostore.tapestry.impl.StaticFederation", init_args);
	return result;
    }
}

