/**
 * RouteMsg.
 *
 * @author     Sean C. Rhea
 * @version    $Id: RouteMsg.java,v 1.9 2002/03/27 01:33:03 srhea Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.Types;
import ostore.util.NodeId;
import ostore.util.QSArray;
import ostore.util.QuickSerializable;
import ostore.util.SecureHash;
import ostore.util.ByteUtils;
import ostore.util.TypeTable;
import ostore.util.QSByteArray;
import ostore.util.QSException;

public class RouteMsg extends NetworkMessage {

    public SecureHash src;
    public SecureHash dest;
    public int hop;
    public QSByteArray mac;
    public boolean prefix_only;

    private QuickSerializable _user_data;
    private int _user_data_length;
    private int _user_data_offset;
    private long _user_data_type;
    private byte [] _user_data_buffer;

    public QuickSerializable user_data () throws QSException {
	if (_user_data == null) {
	    int [] offset = {_user_data_offset};
	    _user_data = (QuickSerializable) TypeTable.new_object (
		    _user_data_type, _user_data_buffer, offset);
	}

	return _user_data;
    }

    public RouteMsg (NodeId peer, SecureHash src, SecureHash dest,
		     boolean prefix_only, int hop, 
		     QuickSerializable user_data) {
	super (peer, false /* outbound */);
	this.src = src;
	this.dest = dest;
	this.prefix_only = prefix_only;
	this.hop = hop;
	_user_data = user_data;
    }

    public RouteMsg (byte [] data, int [] offset) throws QSException {
	super (data, offset);
	int type = ByteUtils.bytesToInt (data, offset);
	src = (SecureHash) TypeTable.new_object (type, data, offset);
	type = ByteUtils.bytesToInt (data, offset);
	dest = (SecureHash) TypeTable.new_object (type, data, offset);
	prefix_only = ByteUtils.bytesToBool (data, offset);
	hop = ByteUtils.bytesToInt (data, offset);
	_user_data_type = ByteUtils.bytesToLong (data, offset);
	_user_data_length = ByteUtils.bytesToInt (data, offset);
	_user_data_buffer = data;
	_user_data_offset = offset [0];
	offset [0] += _user_data_length;
	if (ByteUtils.bytesToBool (data, offset))
	    mac = new QSByteArray (data, offset);
    }

    public QuickSerializable data_to_mac () throws QSException {
	user_data ();
	QuickSerializable [] a = {src, dest, _user_data};
	return new QSArray (a);
    }

    public void to_bytes (byte [] data, int [] offset) {
	super.to_bytes (data, offset);
	ByteUtils.intToBytes (src.type_code (), data, offset);
	src.to_bytes (data, offset);
	ByteUtils.intToBytes (dest.type_code (), data, offset);
	dest.to_bytes (data, offset);
	ByteUtils.boolToBytes (prefix_only, data, offset);
	ByteUtils.intToBytes (hop, data, offset);
	if (_user_data != null) {
	    int [] o = {0};
	    _user_data.to_bytes (null, o);
	    int len = o[0];
	    ByteUtils.longToBytes (
		    TypeTable.type_code (_user_data), data, offset);
	    ByteUtils.intToBytes (len, data, offset);
	    _user_data.to_bytes (data, offset);
	}
	else {
	    ByteUtils.longToBytes (
		    _user_data_type, data, offset);
	    ByteUtils.intToBytes (_user_data_length, data, offset);
	    offset [0] += ByteUtils.memcpy (data, offset [0], 
		    _user_data_buffer, _user_data_offset, _user_data_length);
	}
	if (mac == null)
	    ByteUtils.boolToBytes (false, data, offset);
	else {
	    ByteUtils.boolToBytes (true, data, offset);
	    mac.to_bytes (data, offset);
	}
    }

    public int type_code () {
	throw new NoSuchMethodError ();
	// To test new type code stuff.  --srhea
	// return Types.TAPESTRY_ROUTE_MSG;
    }

    public Object clone () throws CloneNotSupportedException {
	RouteMsg result = (RouteMsg) super.clone ();
	result.src = src;
	result.dest = dest;
	result.prefix_only = prefix_only;
	result.hop = hop;
	result._user_data = _user_data;
	result._user_data_length = _user_data_length;
	result._user_data_offset = _user_data_offset;
	result._user_data_type = _user_data_type;
	result._user_data_buffer = _user_data_buffer;
	result.mac = mac;
	return result;
    }

    public String toString () {
	return "(RouteMsg " + super.toString () + " " + src + "->" + dest + 
	    " hop " + hop + " " +
	    (mac == null ? "without" : "with") + " mac " + _user_data + ")";
    }
}

