/**
 * RouteObjDestMsg.
 *
 * @author     Sean C. Rhea
 * @version    $Id: RouteObjDestMsg.java,v 1.6 2002/03/28 22:57:46 srhea Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.Types;
import ostore.util.NodeId;
import ostore.util.QuickSerializable;
import ostore.util.SecureHash;
import ostore.util.ByteUtils;
import ostore.util.TypeTable;
import ostore.util.QSException;

public class RouteObjDestMsg extends NetworkMessage {

    public SecureHash src;
    public SecureHash dest;
    public SecureHash guid;
    public int hop;

    private QuickSerializable _user_data;
    private int _user_data_length;
    private int _user_data_offset;
    private long _user_data_type;
    private byte [] _user_data_buffer;

   public QuickSerializable user_data () throws QSException {
	if (_user_data == null) {
	    int [] offset = {_user_data_offset};
	    _user_data = (QuickSerializable) TypeTable.new_object (
		    _user_data_type, _user_data_buffer, offset);
	}

	return _user_data;
   }

   public RouteObjDestMsg (NodeId peer, SecureHash src, SecureHash dest,
	    SecureHash guid, int hop, 
	    QuickSerializable user_data) {
	super (peer, false /* outbound */);
	this.src = src;
	this.dest = dest;
	this.guid = guid;
	this.hop = hop;
	_user_data = user_data;
    }

   public RouteObjDestMsg (NodeId peer, SecureHash src, SecureHash dest,
	    SecureHash guid, int hop, int user_data_length, 
	    int user_data_offset, long user_data_type, 
	    byte [] user_data_buffer) {

	super (peer, false /* outbound */);
	this.src = src;
	this.dest = dest;
	this.guid = guid;
	this.hop = hop;
	this._user_data_length = user_data_length;
	this._user_data_offset = user_data_offset;
	this._user_data_type = user_data_type;
	this._user_data_buffer = user_data_buffer;
    }

    public RouteObjDestMsg (byte [] data, int [] offset) throws QSException {
	super (data, offset);
	long type = ByteUtils.bytesToLong (data, offset);
	src = (SecureHash) TypeTable.new_object (type, data, offset);
	type = ByteUtils.bytesToLong (data, offset);
	dest = (SecureHash) TypeTable.new_object (type, data, offset);
	type = ByteUtils.bytesToLong (data, offset);
	guid = (SecureHash) TypeTable.new_object (type, data, offset);
	hop = ByteUtils.bytesToInt (data, offset);
	_user_data_type = ByteUtils.bytesToLong (data, offset);
	_user_data_length = ByteUtils.bytesToInt (data, offset);
	_user_data_buffer = data;
	_user_data_offset = offset [0];
	offset [0] += _user_data_length;
    }

    public void to_bytes (byte [] data, int [] offset) {
	super.to_bytes (data, offset);
	ByteUtils.longToBytes (TypeTable.type_code (src), data, offset);
	src.to_bytes (data, offset);
	ByteUtils.longToBytes (TypeTable.type_code (dest), data, offset);
	dest.to_bytes (data, offset);
	ByteUtils.longToBytes (TypeTable.type_code (guid), data, offset);
	guid.to_bytes (data, offset);
	ByteUtils.intToBytes (hop, data, offset);
	if (_user_data != null) {
	    int [] o = {0};
	    _user_data.to_bytes (null, o);
	    int len = o[0];
	    ByteUtils.longToBytes (
		    TypeTable.type_code (_user_data), data, offset);
	    ByteUtils.intToBytes (len, data, offset);
	    _user_data.to_bytes (data, offset);
	}
	else {
	    ByteUtils.longToBytes (
		    _user_data_type, data, offset);
	    ByteUtils.intToBytes (_user_data_length, data, offset);
	    offset [0] += ByteUtils.memcpy (data, offset [0], 
		    _user_data_buffer, _user_data_offset, _user_data_length);
	}
    }

    public int type_code () {
	return Types.TAPESTRY_ROUTE_OBJ_DEST_MSG;
    }

    public Object clone () throws CloneNotSupportedException {
	RouteObjDestMsg result = (RouteObjDestMsg) super.clone ();
	result.src = src;
	result.dest = dest;
	result.guid = guid;
	result.hop = hop;
	result._user_data = _user_data;
	result._user_data_length = _user_data_length;
	result._user_data_offset = _user_data_offset;
	result._user_data_type = _user_data_type;
	result._user_data_buffer = _user_data_buffer;
	return result;
    }

    public String toString () {
	return "(RouteObjDestMsg " + super.toString () + ")";
    }
}

