/**
 * RouteObjRootMsg.
 *
 * @author     Sean C. Rhea
 * @version    $Id: RouteObjRootMsg.java,v 1.8 2002/03/28 22:57:46 srhea Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.Types;
import ostore.util.NodeId;
import ostore.util.QuickSerializable;
import ostore.util.SecureHash;
import ostore.util.ByteUtils;
import ostore.util.TypeTable;
import ostore.util.QSException;
import ostore.tapestry.api.TapestryQuery;
import ostore.tapestry.api.TapestryQueryState;

public class RouteObjRootMsg extends NetworkMessage {

    public SecureHash src;
    public SecureHash guid;
    public TapestryQuery query;
    public TapestryQueryState query_state;
    public SecureHash id;
    public int hop;
    public NodeId requestor;

    private QuickSerializable _user_data;
    public int _user_data_length;
    public int _user_data_offset;
    public long _user_data_type;
    public byte [] _user_data_buffer;

    public QuickSerializable user_data () throws QSException {
	if (_user_data == null) {
	    int [] offset = {_user_data_offset};
	    _user_data = (QuickSerializable) TypeTable.new_object (
		    _user_data_type, _user_data_buffer, offset);
	}

	return _user_data;
    }

    public RouteObjRootMsg (NodeId peer, SecureHash src, SecureHash guid,
	    TapestryQuery query, TapestryQueryState query_state, 
	    SecureHash id, int hop, QuickSerializable user_data,
	    NodeId requestor) {
	super (peer, false /* outbound */);
	this.src = src;
	this.guid = guid;
	this.query = query;
	this.query_state = query_state;
	this.id = id;
	this.hop = hop;
	this.requestor = requestor;
	_user_data = user_data;
    }

    public RouteObjRootMsg (byte [] data, int [] offset) throws QSException {
	super (data, offset);
	long type = ByteUtils.bytesToLong (data, offset);
	src = (SecureHash) TypeTable.new_object (type, data, offset);
	type = ByteUtils.bytesToLong (data, offset);
	guid = (SecureHash) TypeTable.new_object (type, data, offset);
	type = ByteUtils.bytesToLong (data, offset);
	if (type != 0)
	    query = (TapestryQuery) TypeTable.new_object (type, data, offset);
	type = ByteUtils.bytesToLong (data, offset);
	if (type != 0)
	    query_state = (TapestryQueryState) 
		TypeTable.new_object (type, data, offset);
	type = ByteUtils.bytesToLong (data, offset);
	if (type != 0)
	    id = (SecureHash) TypeTable.new_object (type, data, offset);
	hop = ByteUtils.bytesToInt (data, offset);
	requestor = new NodeId(data, offset);
	_user_data_type = ByteUtils.bytesToLong (data, offset);
	_user_data_length = ByteUtils.bytesToInt (data, offset);
	_user_data_buffer = data;
	_user_data_offset = offset [0];
	offset [0] += _user_data_length;
    }

    public void to_bytes (byte [] data, int [] offset) {
	super.to_bytes (data, offset);
	ByteUtils.longToBytes (TypeTable.type_code (src), data, offset);
	src.to_bytes (data, offset);
	ByteUtils.longToBytes (TypeTable.type_code (guid), data, offset);
	guid.to_bytes (data, offset);
	if (query == null)
	    ByteUtils.longToBytes (0, data, offset);
	else {
	    ByteUtils.longToBytes (TypeTable.type_code (query), data, offset);
	    query.to_bytes (data, offset);
	}
	if (query_state == null)
	    ByteUtils.longToBytes (0, data, offset);
	else {
	    ByteUtils.longToBytes (TypeTable.type_code (query_state), data, offset);
	    query_state.to_bytes (data, offset);
	}
	if (id == null) {
	    ByteUtils.longToBytes (0, data, offset);
	}
	else {
	    ByteUtils.longToBytes (TypeTable.type_code (id), data, offset);
	    id.to_bytes (data, offset);
	}
	ByteUtils.intToBytes(hop, data, offset);
	requestor.to_bytes(data, offset);
   	if (_user_data != null) {
	    int [] o = {0};
	    _user_data.to_bytes (null, o);
	    int len = o[0];
	    ByteUtils.longToBytes (
		    TypeTable.type_code (_user_data), data, offset);
	    ByteUtils.intToBytes (len, data, offset);
	    _user_data.to_bytes (data, offset);
	}
	else {
	    ByteUtils.longToBytes (
		    _user_data_type, data, offset);
	    ByteUtils.intToBytes (_user_data_length, data, offset);
	    offset [0] += ByteUtils.memcpy (data, offset [0], 
		    _user_data_buffer, _user_data_offset, _user_data_length);
	}
    }

    public int type_code () {
	return Types.TAPESTRY_ROUTE_OBJ_ROOT_MSG;
    }

    public Object clone () throws CloneNotSupportedException {
	RouteObjRootMsg result = (RouteObjRootMsg) super.clone ();
	result.src = src;
	result.guid = guid;
	result.query = query;
	result.query_state = query_state;
	result.id = id;
	result.hop = hop;
	result.requestor = requestor;
	result._user_data = _user_data;
	result._user_data_length = _user_data_length;
	result._user_data_offset = _user_data_offset;
	result._user_data_type = _user_data_type;
	result._user_data_buffer = _user_data_buffer;
	return result;
    }

    public String toString () {
	return "(RouteObjRootMsg " + super.toString () + ")";
    }
}

