/**
 * StaticAllNodesMsg.
 *
 * @author     Sean C. Rhea
 * @version    $Id: StaticAllNodesMsg.java,v 1.1 2001/09/11 02:40:55 srhea Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.Types;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.ByteUtils;
import ostore.util.QSException;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

public class StaticAllNodesMsg extends NetworkMessage {

    public StaticAllNodesMsg (NodeId peer, Map nodes) {
	super (peer, false /* outbound */);
	_nodes = nodes;
    }

    public StaticAllNodesMsg (byte [] buffer, int [] offset) 
    throws QSException {
	super (buffer, offset);
	int count = ByteUtils.bytesToInt (buffer, offset);
	_nodes = new HashMap ();
	for (int i = 0; i < count; ++i) {
	    NodeId n = new NodeId (buffer, offset);
	    int typecode = ByteUtils.bytesToInt (buffer, offset);
	    SecureHash g = (SecureHash) TypeTable.new_object (
		    typecode, buffer, offset);
	    _nodes.put (n, g);
	}
    }

    public Map nodes () { return _nodes; }

    public void to_bytes (byte [] buffer, int [] offset) {
	super.to_bytes (buffer, offset);
	ByteUtils.intToBytes (_nodes.size (), buffer, offset);
	for (Iterator i = _nodes.keySet ().iterator (); i.hasNext (); ) {
	    NodeId n = (NodeId) i.next ();
	    SecureHash g = (SecureHash) _nodes.get (n);
	    n.to_bytes (buffer, offset);
	    ByteUtils.intToBytes (g.type_code (), buffer, offset);
	    g.to_bytes (buffer, offset);
	}
    }

    public int type_code () {
	return Types.TAPESTRY_STATIC_ALL_NODES_MSG;
    }

    public Object clone () throws CloneNotSupportedException {
	StaticAllNodesMsg result = (StaticAllNodesMsg) super.clone ();
	result._nodes = _nodes;
	return result;
    }

    public String toString () {
	// TODO
	return "(StaticAllNodesMsg " + super.toString () + ")";
    }

    public Map _nodes;
}

