/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.util;

/**
 * A convenience class for generating error messages, complete with "carp" and
 * "croak", borrowed from PERL.
 *
 * @author	Dennis Geels
 * @version	$Id: Carp.java,v 1.9 2002/04/05 20:29:23 geels Exp $
 */
public abstract class Carp
{
    ////////////////////       CONSTANTS	////////////////////
    ////////////////////    STATIC VARIABLES 	////////////////////
    ////////////////////       VARIABLES	////////////////////
    ////////////////////  STATIC CONSTRUCTORS	////////////////////
    ////////////////////      CONSTRUCTORS	////////////////////
    ////////////////////     STATIC METHODS 	////////////////////

    /**
     * Print a message to standard out.
     */
    public static void out( String msg ) {
	System.out.println(  msg );
    }

    /**
     * Print an error message to standard error.
     */
    public static void carp( String msg ) {
	System.err.println( "ERROR:\t" + msg );
    }

    /**
     * Print an exception's message to standard error.
     */
    public static void carp( Throwable e ) {
	carp( "Caught " + e.getClass().getName() );
	carp( e.getMessage() );
    }

    /**
     * Print an error message and current stack trace to standard error.
     */
    public static void error( String msg ) {
	carp( msg );
        Thread.dumpStack();
    }

    /**
     * Print an exception's message and stack trace to standard error.
     */
    public static void error( Throwable e ) {
	carp( e );
	e.printStackTrace();
    }

    /**
     * Kill the whole jvm.
     */
    public static void croak() {
	System.exit(-1);
    }

    /**
     * Print an error message to standard error and exit.
     */
    public static void croak( String msg ) {
	carp( msg );
	croak();
    }

    /**
     * Print an exception's message to standard error and exit.
     */
    public static void croak( Throwable e ) {
	carp( e );
	croak();
    }
    /**
     * Exit with an error code.
     */
    public static void die( ) {
	Thread.dumpStack();
	croak();
    }

    /**
     * Print an error message and stack trace to standard error and exit.
     */
    public static void die( String msg ) {
	carp( msg );
	die();
    }

    /**
     * Print an exception's message and stack trace to standard error and exit.
     */
    public static void die( Throwable e ) {
	carp( e );
	die();
    }

    /**
     * Print out the stack trace of the current thread.  The stack trace will
     * include the call to <code>Carp.dumpStack</code>.
     */
    public static void dumpStack() {
	Thread.dumpStack();
    }
    
    //////////////////// 	METHODS		////////////////////
    ////////////////////     INNER CLASSES	////////////////////
}
