/**
 * Digestible.java
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.util;

/**
 * An interface for classes which understand how to use
 * {@link SecureHashBuffer}s to hash themselves.
 * <p>
 * The default method for hashing <code>QuickSerializable</code> objects is
 * to serialize the object and hash the resulting byte array.
 * <code>SecureHashBuffer</code>s allow objects to incrementally update the
 * hash's internal digest, eliminating much of the copying involved in
 * serialization.  The reduced overhead should be noticeable for large,
 * mostly pre-serialized objects, such as data blocks.
 *
 * @author	Dennis Geels
 * @version	$Id: Digestible.java,v 1.3 2002/03/29 00:24:00 srhea Exp $
 */

public interface Digestible extends QuickSerializable
{
    //////////////////// 	METHODS		////////////////////

    /**
     * Adds this object's state to the specified
     * <code>SecureHashBuffer</code> using the various <code>add</code>
     * methods.
     *
     * @see SecureHashBuffer#add( byte )
     * @see SecureHashBuffer#add( byte[] )
     * @see SecureHashBuffer#add( byte[], int, int )
     * @see SecureHashBuffer#add( Digestible )
     */
    void add_to_digest( SecureHashBuffer buffer );
}
