/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.util;

/**
***  QSByteArray.java
***
***  @author  Gifford Cheung
***  @version $Id: QSByteArray.java,v 1.3 2002/03/29 01:59:58 srhea Exp $
**/
public class QSByteArray implements Digestible
{
    private byte[] _bytes;

	public QSByteArray( byte[] bytes) {
		_bytes = bytes;
	}

    public QSByteArray(byte [] data, int [] offset)
    {
        int length = ByteUtils.bytesToInt( data, offset);
		_bytes = new byte[length];
		offset[0] += ByteUtils.memcpy( _bytes, 0, data, offset[0], length);
		return;
    }

    public byte[] bytes()
    {
        return _bytes;
    }

    //  methods for ostore.util.QuickSerializable interface
    public void to_bytes(byte [] data, int [] offset)
    {
	ByteUtils.intToBytes(_bytes.length, data, offset);
	offset[0] += ByteUtils.memcpy( data, offset[0], 
		_bytes, 0, _bytes.length);
    }

    public void add_to_digest( SecureHashBuffer buffer ) {
	buffer.add (_bytes.length);
	buffer.add (_bytes);
    }

    public int type_code() 
    {
        return ostore.util.Types.QSBYTEARRAY;
    }
    //  end methods for ostore.util.QuickSerializable interface

    public Object clone()
        throws CloneNotSupportedException
    {
        return super.clone();
    }

    public boolean equals(Object other)
    {
        if (! (other instanceof QSByteArray))
        {
            return false;
        }

        return ByteUtils.equals(_bytes,((QSByteArray) other).bytes());
    }

}
