/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.util;

/**
***  @author  Patrick R. Eaton
***  @version $Id: QSLong.java,v 1.6 2002/03/29 21:14:53 eaton Exp $
**/
public class QSLong implements QuickSerializable, Digestible
{
    private long _value;

    public QSLong()
    {
        _value = 0;
        return;
    }

    public QSLong(long l)
    {
        _value = l;
        return;
    }

    public QSLong(Long l)
    {
        _value = l.longValue();
        return;
    }

    public QSLong(byte [] data, int [] offset)
    {
        _value = ByteUtils.bytesToLong(data, offset);
        return;
    }

    public long longValue()
    {
        return _value;
    }

    //  methods for ostore.util.QuickSerializable interface
    public void to_bytes(byte [] data, int [] offset)
    {
        ByteUtils.longToBytes(_value, data, offset);
    }

    public int type_code() 
    {
        return ostore.util.Types.QSLONG;
    }
    //  end methods for ostore.util.QuickSerializable interface

    public void add_to_digest(SecureHashBuffer buffer)
    {
        buffer.add(_value);
        return;
    }

    //  methods for Object
    public Object clone()
        throws CloneNotSupportedException
    {
        return super.clone();
    }

    public boolean equals(Object other)
    {
        if (! (other instanceof QSLong))
        {
            return false;
        }

        return (_value == ((QSLong) other).longValue());
    }

    public int hashCode()
    {
        return ((int) (_value % Integer.MAX_VALUE));
    }

    public String toString()
    {
        return Long.toString(_value);
    }
    //  end methods for Object
}
