/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.util;

/**
 * A simple Queue.
 *
 * @author     Sean C. Rhea
 * @version    $Id: Queue.java,v 1.3 2001/10/18 00:20:03 eaton Exp $
 */
public class Queue {

    private class Node {
	Object data;
	Node next;
    }

    private Node _head, _tail;
    private int _size;

    public Queue () {
	_head = _tail = null;
	_size = 0;
    }

    public synchronized Object head () {
	if (_head == null) 
	    return null;
	return _head.data;
    }

    public synchronized boolean empty () {
	return (_head == null);
    }

    public synchronized int size () {
	return _size;
    }

    public synchronized void enqueue (Object object) {
	if (_head == null) {
	    _head = _tail = new Node ();
	    _head.data = object;
	}
	else {
	    Node node = new Node ();
	    node.data = object;
	    _tail.next = node;
	    _tail = node;
	}
	_size += 1;
    }

    public synchronized Object dequeue () {
	if (_head == null) 
	    return null;
	Object result = _head.data;
	_head = _head.next;
	_size -= 1;
	return result;
    }

    public static void main (String [] argv) {
	Queue q = new Queue ();
	Assert.assert (q.empty (), "1) Oops!");
	for (int i = 0; i < 100; ++i)
	    q.enqueue (new Integer (i));
	Assert.assert (! q.empty (), "2) Oops!");
	for (int i = 0; i < 100; ++i)
	    Assert.assert (((Integer) q.dequeue ()).intValue () == i, 
	                   "3) Oops!");
	Assert.assert (q.empty (), "4) Oops!");
    }
}

