/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.util;

import ostore.util.*;

/**
***  @author  Patrick R. Eaton
***  @version $Id: Timestamp.java,v 1.3 2001/10/18 00:20:03 eaton Exp $
**/
public class Timestamp implements QuickSerializable
{
    private long _time;

    public Timestamp()
    {
        _time = 0;

        return;
    }

    public Timestamp (byte buffer[], int [] offset)
    {
        Assert.assert(buffer != null,
                      "Timestamp.from_bytes:  null buffer");
        Assert.assert(offset [0] >= 0,
                      "Timestamp.from_bytes:  negative offset");

        //  get the time
        _time = ByteUtils.bytesToLong(buffer, offset);
    }

    private Timestamp (long t) {
	_time = t;
    }

    public static Timestamp currentTime () {
	return new Timestamp (System.currentTimeMillis ());
    }

    public boolean precedes (Timestamp rhs) {
	return _time <= rhs._time;
    }

    public long getTime()
    {
        return _time;
    }

    public boolean equals(Object other)
    {
        if (other == null)
        {
            return false;
        }

        Timestamp t = (Timestamp) other;

        return (_time == t.getTime());
    }

    public String toString()
    {
        String s = new String();

        s += "(Time ";
        s += Long.toString(_time);
        s += ")";

        return s;
    }

    //  implementation of the ostore.util.QuickSerializable interface
    public void to_bytes(byte buffer[], int [] offset)
    {
        Assert.assert(offset [0] >= 0,
                      "Timestamp.to_bytes:  negative offset");

        //  put the time
        ByteUtils.longToBytes(_time, buffer, offset);
    }

    public int type_code()
    {
        return ostore.util.Types.TIMESTAMP;
    }
    //  end implementation of the ostore.util.QuickSerializable interface
}
