#!/usr/bin/perl -w
#
# $Id: cfgcat.pl,v 1.5 2002/03/14 18:52:53 geels Exp $
#
# Concatinates several config files desiged to run on the same machine into
# one.  Allows for running multiple "virtual" OceanStore servers on a
# single physical machine.  Pretty ghetto at the moment, but works.  :)
#
# usage:  cfgcat.pl <config files>
#
# (Output is on STDOUT.)
#
# See the new test-ostore.inner for an example usage.
#
use strict;

my $i = 0;
while ($i <= $#ARGV) {
    # for each input .cfg file
    my $thisnodeid;

    open (IN, "<$ARGV[$i]");

    while (<IN>) {
	# find the node id
	if (m/^[^\#]*node_id\s+(\S+)$/) {
	    $thisnodeid = $1;
	}

	# print the header of the first file only
	if ($i == 0) {
	    print; 
	}

	# stop after stages
	if (m/^[^\#]*<stages>/) {
	    last;
	}
    }

    while (1) {
	my $stage;
	my $done = 0;

	if( ! defined( $thisnodeid ) ) {
	    die "ERROR: no global \"node_id\" field in $ARGV[$i]\n";
	}
	
        # find each stage
	while (<IN>) {
	    if (m/^[^\#]*<\/stages>/) {
		$done = 1;
		last;
	    }
	    if (m/^([^\#]*)<([^\/]+)>(.*)$/) {
		# append the NodeId to the stage name
		$stage = $2;
		print "$1<$stage-$thisnodeid>$3\n";
		last;
	    }
	    else {
		print;
	    }
	}
	
	if ($done == 0) {
	    while (<IN>) {
		if (m/^([^\#]*)<initargs>/) {
		    # insert NodeId
		    print; 
		    print "\t\tnode_id $thisnodeid\n";
		}
		elsif (m/^([^\#]*)<\/$stage>(.*)$/) {
		    # append the NodeId to the stage name
		    print "$1</$stage-$thisnodeid>$2\n";
		    last;
		}
		else {
		    print;
		}
	    }
	}

	if (! defined( $stage ) ) {
	    last;
	}
    }
    undef $thisnodeid;
    close (IN);

    ++$i;
}

print "\n";
print "    </stages>\n";
print "</sandstorm>\n";

