/*
 * EventQueue.java
 *
 * Copyright (c) 2001 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package soss.core;
import ostore.util.Assert;
import ostore.util.Pair;
import ostore.util.PriorityQueue;
import ostore.dispatch.Classifier;
import mdw.sandStorm.api.QueueElementIF;
import mdw.sandStorm.api.SinkException;

/**
 * The simulator's event queue.
 *
 * @author Sean C. Rhea
 * @version $Id: EventQueue.java,v 1.5 2001/12/19 00:15:35 srhea Exp $
 */
public class EventQueue {

    private static final String class_tag = "EventQueue";
    private static final long TIME_SCALE = 1000 * 1000; // microseconds

    public static EventQueue instance = null;

    public EventQueue () {
	_pq = new PriorityQueue (1000 * 1000);
	instance = this;
    }

    /**
     * Schedule the <code>event</code> to be dispatched on the given
     * <code>queue</code> at the given <code>time</code>.
     */
    public void schedule (QueueElementIF event, double time, 
	    Classifier queue) {
	final String tag = class_tag + ".schedule";

	Assert.assert (time >= _current_time, 
		tag + ": time (" + time + ") must be >= current_time (" + 
		_current_time + ")");

	long ltime = (long) (time * TIME_SCALE);
	/*System.out.println ("EventQueue.schedule: scheduling " +
		event.getClass ().getName () + " event " +
		"for node " + queue.node_id () + " at time " + 
		time + ", modified = " + ltime + ".");*/

	_pq.insert (new Pair (event, queue), ltime, false);
    }

    public void run () {
	final String tag = class_tag + ".run";

	while (! _pq.empty ()) {
	    long priority = _pq.topPriority (false);
	    Pair p = (Pair) _pq.extract (false);
	    _current_time = ((double) priority) / TIME_SCALE;

	    QueueElementIF event = (QueueElementIF) p.first;
	    Classifier classifier = (Classifier) p.second;

	    /*System.out.println ("EventQueue.run: dispatching " +
		    event + " to node [" + classifier.node_id ().port () + 
		    "] at time " + _current_time + ".");*/

	    try {
		classifier.dispatch (event);
	    } catch (SinkException e) {
		Assert.assert (false, tag + ": could not dispatch " + event);
	    }
	}
    }

    public double current_time () { return _current_time; }

    private PriorityQueue _pq;
    private double _current_time;
}
	
