/*
 * Main.java
 *
 * Copyright (c) 2001 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package soss.core;
import mdw.sandStorm.main.SandstormConfig;
import mdw.sandStorm.api.StageIF;
import mdw.sandStorm.api.EventHandlerIF;
import mdw.sandStorm.api.ConfigDataIF;
import mdw.sandStorm.api.ManagerIF;
import mdw.sandStorm.api.NoSuchStageException;
import mdw.sandStorm.api.SignalMgrIF;
import mdw.sandStorm.api.ProfilerIF;
import java.util.HashMap;
import java.util.Map;

public class Manager implements ManagerIF {

    public Manager (SandstormConfig config) {
	_config = config;
	_stages = new HashMap ();
    }

    public void addStage(String stagename, StageIF stage) {
	_stages.put (stagename, stage);
    }

    public StageIF getStage(String stagename) throws NoSuchStageException {
	StageIF result = (StageIF) _stages.get (stagename);
	if (result == null)
	    throw new NoSuchStageException ();
	return result;
    }

    public StageIF createStage(String stagename, EventHandlerIF eventHandler,
	    String initargs[]) throws Exception {
	throw new IllegalArgumentException ();
    }

    /**
     * Returns a handle to the system signal interface.
     */
    public SignalMgrIF getSignalMgr() {
	throw new IllegalArgumentException ();
    }

    /**
     * Returns a handle to the system profiler. 
     */
    public ProfilerIF getProfiler() {
	throw new IllegalArgumentException ();
    }

    /**
     * Returns a copy of the SandstormConfig for this Manager. This
     * contains all of the global options used by the runtime system. Note
     * that modifying any options of this copy does not in fact change the
     * runtime parameters of the system; this is used for informational
     * purposes only.
     */
    public SandstormConfig getConfig() {
	return _config;
    }

    private Map _stages;
    private SandstormConfig _config;
}


